@@ -55,9 +55,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -72,9 +72,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -82,9 +82,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -115,25 +115,11 @@ typedef enum
 }
 TSG_Tool_Error;
 
-//---------------------------------------------------------
-typedef enum
-{
-	TOOL_SCRIPT_CMD_SHELL	= 0,
-	TOOL_SCRIPT_CMD_BATCH,
-	TOOL_SCRIPT_CHAIN,
-	TOOL_SCRIPT_PYTHON,
-	TOOL_SCRIPT_PYTHON_WRAP_NAME,
-	TOOL_SCRIPT_PYTHON_WRAP_NAME_CALL,
-	TOOL_SCRIPT_PYTHON_WRAP_ID,
-	TOOL_SCRIPT_PYTHON_WRAP_ID_CALL
-}
-TSG_Tool_Script_Type;
-
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //						CSG_Tool						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -149,6 +135,7 @@ class SAGA_API_DLL_EXPORT CSG_Tool
 {
 	friend class CSG_Tool_Interactive_Base;
 	friend class CSG_Tool_Library_Interface;
+	friend class CSG_Tool_Library_Manager;
 	friend class CSG_Tool_Chain;
 
 public:
@@ -175,10 +162,12 @@ public:
 	virtual CSG_String			Get_MenuPath				(void)	{	return( SG_T("") );	}
 	virtual CSG_String			Get_MenuPath				(bool bSolved);
 
-	int							Get_Parameters_Count		(void)	{	return( m_npParameters );	}
 	CSG_Parameters *			Get_Parameters				(void)	{	return( &Parameters );	}
-	CSG_Parameters *			Get_Parameters				(int i)	{	return( i >= 0 && i < m_npParameters ? m_pParameters[i] : NULL );	}
-	CSG_Parameters *			Get_Parameters				(const CSG_String &Identifier);
+
+	int							Get_Parameters_Count		(void)  const { return( (int)m_pParameters.Get_Size() ); }
+	CSG_Parameters *			Get_Parameters				(int i) const { return( i >= 0 && i < Get_Parameters_Count() ? (CSG_Parameters *)m_pParameters[i] : NULL ); }
+	CSG_Parameters *			Get_Parameters				(int i)       { return( i >= 0 && i < Get_Parameters_Count() ? (CSG_Parameters *)m_pParameters[i] : NULL ); }
+	CSG_Parameters *			Get_Parameters				(const CSG_String &Identifier) const;
 
 	CSG_Parameter *				Get_Parameter				(const CSG_String &ID)	const	{	return( Parameters(ID) );	}
 	CSG_Parameter *				Get_Parameter				(const char       *ID)	const	{	return( Parameters(ID) );	}
@@ -228,6 +217,7 @@ public:
 
 	virtual bool				needs_GUI					(void)	const	{	return( false );	}
 	bool						has_GUI						(void)	const;
+	bool						has_CMD						(void)	const;
 
 	virtual bool				is_Grid						(void)	const	{	return( false );	}
 	virtual bool				is_Interactive				(void)	const	{	return( false );	}
@@ -243,7 +233,21 @@ public:
 
 	const SG_Char *				Get_Execution_Info			(void)	const	{	return( m_Execution_Info );	}
 
-	CSG_String					Get_Script					(TSG_Tool_Script_Type Type, bool bHeader, bool bAllParameters = false);
+	CSG_MetaData				Get_History					(int Depth = -1);
+	bool						Set_History					(CSG_Data_Object *pDataObject, int Depth = -1);
+
+
+	//-----------------------------------------------------
+	enum class Script_Format
+	{
+		CMD_Shell, CMD_Batch, CMD_Usage,
+		Toolchain,
+		Python,
+		Python_Wrapper_Func_Name, Python_Wrapper_Func_ID,
+		Python_Wrapper_Call_Name, Python_Wrapper_Call_ID
+	};
+
+	CSG_String					Get_Script					(Script_Format Format, bool bHeader = true, int Arguments = 0, bool bWrapArgs = true);
 
 
 protected:
@@ -272,8 +276,10 @@ protected:
 
 	//-----------------------------------------------------
 	CSG_Parameters *			Add_Parameters				(const CSG_String &Identifier, const CSG_String &Name, const CSG_String &Description);
-	bool						Dlg_Parameters				(const CSG_String &Identifier);
-	bool						Dlg_Parameters				(CSG_Parameters *pParameters, const CSG_String &Caption);
+	bool						Dlg_Parameters				(const CSG_String &Identifier); // deprecated
+
+	bool						Dlg_Parameters				(CSG_Parameters *pParameters, const CSG_String &Caption = "");
+	bool						Dlg_Parameters				(CSG_Parameters  &Parameters, const CSG_String &Caption = "");
 
 	//-----------------------------------------------------
 	virtual bool				Set_Progress				(int    Position, int    Range     )	const;
@@ -297,79 +303,90 @@ protected:
 	//-----------------------------------------------------
 	bool						DataObject_Add				(CSG_Data_Object *pDataObject, bool bUpdate = false);
 
-	void						DataObject_Update_All		(void);
+	bool						DataObject_Update_All		(void);
 
 	bool						DataObject_Set_History		(CSG_Parameter *pParameter, CSG_MetaData *pHistory = NULL);
 
-	bool						Get_Projection				(CSG_Projection &Projection)	const;
+	bool						Get_Projection				(CSG_Projection &Projection) const;
 
 
-public:	// static functions...
+protected: // static functions...
 
-	static bool					Process_Get_Okay			(bool bBlink = false);
-	static void					Process_Set_Text			(const CSG_String &Text);
-	static void					Process_Set_Text			(const char    *Format, ...);
-	static void					Process_Set_Text			(const wchar_t *Format, ...);
+	static int					_On_Parameter_Changed				(CSG_Parameter *pParameter, int Flags);
 
-	static bool					DataObject_Update			(CSG_Data_Object *pDataObject                                , int Show = SG_UI_DATAOBJECT_UPDATE);
-	static bool					DataObject_Update			(CSG_Data_Object *pDataObject, double Minimum, double Maximum, int Show = SG_UI_DATAOBJECT_UPDATE);
+	static bool					Process_Get_Okay					(bool bBlink = false);
+	static void					Process_Set_Text					(const CSG_String &Text);
+	static void					Process_Set_Text					(const char    *Format, ...);
+	static void					Process_Set_Text					(const wchar_t *Format, ...);
 
-	static bool					DataObject_Get_Colors		(CSG_Data_Object *pDataObject, CSG_Colors &Colors);
-	static bool					DataObject_Set_Colors		(CSG_Data_Object *pDataObject, const CSG_Colors &Colors);
-	static bool					DataObject_Set_Colors		(CSG_Data_Object *pDataObject, int nColors, int Palette = SG_COLORS_DEFAULT, bool bRevert = false);
+	static bool					DataObject_Update					(CSG_Data_Object *pDataObject                                , int Show = SG_UI_DATAOBJECT_UPDATE);
+	static bool					DataObject_Update					(CSG_Data_Object *pDataObject, double Minimum, double Maximum, int Show = SG_UI_DATAOBJECT_UPDATE);
 
-	static bool					DataObject_Get_Parameters	(CSG_Data_Object *pDataObject, CSG_Parameters &Parameters);
-	static bool					DataObject_Set_Parameters	(CSG_Data_Object *pDataObject, CSG_Parameters &Parameters);
-	static bool					DataObject_Set_Parameters	(CSG_Data_Object *pDataObject, CSG_Data_Object *pCopy);
+	static bool					DataObject_Get_Colors				(CSG_Data_Object *pDataObject, CSG_Colors &Colors);
+	static bool					DataObject_Set_Colors				(CSG_Data_Object *pDataObject, const CSG_Colors &Colors);
+	static bool					DataObject_Set_Colors				(CSG_Data_Object *pDataObject, int nColors, int Colors = SG_COLORS_DEFAULT, bool bRevert = false);
 
-	static CSG_Parameter *		DataObject_Get_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, CSG_Parameter *pParameter);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, CSG_Data_Object *pCopy, const CSG_String &ID);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID, int            Value);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID, double         Value);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID, void          *Value);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID, const SG_Char *Value);
-	static bool					DataObject_Set_Parameter	(CSG_Data_Object *pDataObject, const CSG_String &ID, double loVal, double hiVal);	// Range Parameter
+	static bool					DataObject_Get_Parameters			(CSG_Data_Object *pDataObject, CSG_Parameters &Parameters);
+	static bool					DataObject_Set_Parameters			(CSG_Data_Object *pDataObject, CSG_Parameters &Parameters);
+	static bool					DataObject_Set_Parameters			(CSG_Data_Object *pDataObject, CSG_Data_Object *pCopy);
+
+	static CSG_Parameter *		DataObject_Get_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, CSG_Parameter *pParameter);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, CSG_Data_Object *pCopy, const CSG_String &ID);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID, int            Value);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID, double         Value);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID, void          *Value);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID, const SG_Char *Value);
+	static bool					DataObject_Set_Parameter			(CSG_Data_Object *pDataObject, const CSG_String &ID, double loVal, double hiVal); // Range Parameter
+
+	static bool					DataObject_Set_Stretch_Linear		(CSG_Data_Object *pDataObject, int Field = -1, double Minimum = 5., double Maximum = 95.    , int Interval_Mode = 0, double Interval_Log = 1., int Colors = -1, bool bUpdateNow = false);
+	static bool					DataObject_Set_Stretch_StdDev		(CSG_Data_Object *pDataObject, int Field = -1, double StdDev = 2., bool bKeepInRange = false, int Interval_Mode = 0, double Interval_Log = 1., int Colors = -1, bool bUpdateNow = false);
+	static bool					DataObject_Set_Stretch_Percentile	(CSG_Data_Object *pDataObject, int Field = -1, double Minimum = 2., double Maximum = 98.    , int Interval_Mode = 0, double Interval_Log = 1., int Colors = -1, bool bUpdateNow = false);
+	static bool					DataObject_Set_Stretch				(CSG_Data_Object *pDataObject, int Field     , double Minimum     , double Maximum          , int Interval_Mode = 0, double Interval_Log = 1., int Colors = -1, bool bUpdateNow = false);
+
+	static bool					DataObject_Classify_Unique			(CSG_Data_Object *pDataObject, int Field = -1, int Classes_Max = 1024                                   , int Colors = -1);
+	static bool					DataObject_Classify_Equal			(CSG_Data_Object *pDataObject, int Field = -1, int Classes = 10                                         , int Colors = -1);
+	static bool					DataObject_Classify_Defined			(CSG_Data_Object *pDataObject, int Field     , double Interval, bool bOffset = false, double Offset = 0., int Colors = -1);
+	static bool					DataObject_Classify_Quantile		(CSG_Data_Object *pDataObject, int Field = -1, int Classes = 10, bool bHistogram = true                 , int Colors = -1);
+	static bool					DataObject_Classify_Geometric		(CSG_Data_Object *pDataObject, int Field = -1, int Classes = 10, bool bIncreasing = true                , int Colors = -1);
+	static bool					DataObject_Classify_Natural			(CSG_Data_Object *pDataObject, int Field = -1, int Classes = 10                                         , int Colors = -1);
+	static bool					DataObject_Classify_StdDev			(CSG_Data_Object *pDataObject, int Field = -1, double StdDev = 0.5, double StdDev_Max = 4.5             , int Colors = -1);
 
 
 private:
 
-	bool						m_bExecutes, m_bError_Ignore, m_bShow_Progress, m_bWithGUI;
+	bool						m_bExecutes, m_bError_Ignore, m_bShow_Progress, m_bGUI, m_bCMD;
 
-	int							m_npParameters;
-
-	CSG_Array					m_Settings_Stack;
-
-	CSG_Parameters				**m_pParameters;
+	CSG_Array_Pointer			m_pParameters;
 
 	CSG_String					m_ID, m_Library, m_Library_Menu, m_File_Name, m_Author, m_Version, m_Execution_Info;
 
 
 	bool						_Synchronize_DataObjects	(void);
 
-	CSG_String					_Get_Script_CMD				(bool bHeader, bool bAllParameters, TSG_Tool_Script_Type Type);
-	void						_Get_Script_CMD				(CSG_String &Script, CSG_Parameters *pParameters, bool bAllParameters, TSG_Tool_Script_Type Type);
+	CSG_String					_Get_Script_CMD				(bool bHeader                                   , bool bAllArguments, bool bWrapArgs, bool bBatch);
+	void						_Get_Script_CMD				(CSG_String &Script, CSG_Parameters *pParameters, bool bAllArguments, bool bWrapArgs, bool bBatch);
 
-	CSG_String					_Get_Script_Python			(bool bHeader, bool bAllParameters);
-	void						_Get_Script_Python			(CSG_String &Script, CSG_Parameters *pParameters, bool bAllParameters, const CSG_String &Prefix = "");
+	CSG_String					_Get_Script_CMD_Usage		(void);
+	void						_Get_Script_CMD_Usage		(CSG_Parameters *pParameters, class wxCmdLineParser &Parser);
 
-	CSG_String					_Get_Script_Python_Wrap		(bool bHeader, bool bName, bool bCall);
-	bool						_Get_Script_Python_Wrap		(const CSG_Parameter &Parameter, int Constraint, CSG_String &Arguments, CSG_String &Description, CSG_String &Code, bool bCall, const CSG_String &Prefix = "");
+	CSG_String					_Get_Script_Python			(bool bHeader, bool bAllArguments);
+	void						_Get_Script_Python			(CSG_String &Script, CSG_Parameters *pParameters, bool bAllArguments, const CSG_String &Prefix = "");
+
+	CSG_String					_Get_Script_Python_Wrap		(bool bHeader, bool bName, bool bCall, int AllArguments, bool bWrapArgs);
+	bool						_Get_Script_Python_Wrap		(const CSG_Parameter &Parameter, int Constraint, CSG_String &Arguments, CSG_String &Description, CSG_String &Code, bool bCall, bool bOnlyNonDefaults, bool bWrapArgs, const CSG_String &Prefix = "");
 
-	CSG_MetaData				_Get_Output_History			(void);
 	void						_Set_Output_History			(void);
 
 	void						_Update_Parameter_States	(CSG_Parameters *pParameters);
 
-	static int					_On_Parameter_Changed		(CSG_Parameter *pParameter, int Flags);
-
 };
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					CSG_Tool_Grid						 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -441,9 +458,9 @@ private:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //				CSG_Tool_Interactive_Base				 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -544,9 +561,9 @@ private:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //				CSG_Tool_Interactive					 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -570,9 +587,9 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //				CSG_Tool_Grid_Interactive				 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -604,9 +621,9 @@ protected:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //			Tool Library Interface Definitions			 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -645,7 +662,7 @@ public:
 	int							Get_Count				(void);
 	CSG_Tool *					Get_Tool				(int i);
 
-	CSG_Tool *					Create_Tool				(int i, bool bWithGUI = false);
+	CSG_Tool *					Create_Tool				(int i, bool bWithGUI = false, bool bWithCMD =true);
 	bool						Delete_Tool				(CSG_Tool *pTool);
 	bool						Delete_Tools			(void);
 
@@ -674,26 +691,27 @@ typedef CSG_Tool_Library_Interface *	(* TSG_PFNC_TLB_Get_Interface)	(void);
 #define TLB_INTERFACE_SKIP_TOOL			((CSG_Tool *)0x1)
 
 //---------------------------------------------------------
-#define TLB_INTERFACE_INITIALIZE	CSG_Tool_Library_Interface	TLB_Interface;\
+#define TLB_INTERFACE_ESTABLISH CSG_Tool_Library_Interface TLB_Interface;\
 \
 extern "C" _SAGA_DLL_EXPORT CSG_Tool_Library_Interface *	TLB_Get_Interface   (void)\
 {\
 	return( &TLB_Interface );\
 }\
-\
-extern "C" _SAGA_DLL_EXPORT bool TLB_Initialize	(const SG_Char *TLB_Path)\
+
+//---------------------------------------------------------
+#define TLB_INTERFACE_INITIALIZE extern "C" _SAGA_DLL_EXPORT bool TLB_Initialize(const SG_Char *TLB_Path)\
 {\
 	return( TLB_Interface.Create(SAGA_VERSION, TLB_Path, Get_Info, Create_Tool) );\
 }\
 
 //---------------------------------------------------------
-#define TLB_INTERFACE_FINALIZE		extern "C" _SAGA_DLL_EXPORT bool TLB_Finalize	(void)\
+#define TLB_INTERFACE_FINALIZE   extern "C" _SAGA_DLL_EXPORT bool TLB_Finalize  (void)\
 {\
 	return( true );\
 }\
 
 //---------------------------------------------------------
-#define TLB_INTERFACE	TLB_INTERFACE_INITIALIZE TLB_INTERFACE_FINALIZE
+#define TLB_INTERFACE	TLB_INTERFACE_ESTABLISH TLB_INTERFACE_INITIALIZE TLB_INTERFACE_FINALIZE
 
 //---------------------------------------------------------
 #ifndef SWIG
@@ -704,9 +722,9 @@ extern CSG_Tool_Library_Interface	TLB_Interface;
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
