@@ -54,9 +54,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -70,9 +70,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -82,9 +82,9 @@ CSG_Tool_Chain::CSG_Tool_Chain(void)
 }
 
 //---------------------------------------------------------
-CSG_Tool_Chain::CSG_Tool_Chain(const CSG_Tool_Chain &Tool, bool bWithGUI)
+CSG_Tool_Chain::CSG_Tool_Chain(const CSG_Tool_Chain &Tool, bool bWithGUI, bool bWithCMD)
 {
-	Create(Tool, bWithGUI);
+	Create(Tool, bWithGUI, bWithCMD);
 }
 
 //---------------------------------------------------------
@@ -126,11 +126,11 @@ void CSG_Tool_Chain::Set_Library_Menu(const CSG_String &Menu)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CSG_Tool_Chain::Create(const CSG_Tool_Chain &Tool, bool bWithGUI)
+bool CSG_Tool_Chain::Create(const CSG_Tool_Chain &Tool, bool bWithGUI, bool bWithCMD)
 {
 	if( !Create(Tool.m_Chain) )
 	{
@@ -141,7 +141,8 @@ bool CSG_Tool_Chain::Create(const CSG_Tool_Chain &Tool, bool bWithGUI)
 	m_Library      = Tool.m_Library;
 	m_Library_Menu = Tool.m_Library_Menu;
 	m_File_Name	   = Tool.m_File_Name;
-	m_bWithGUI     = bWithGUI;
+	m_bGUI         = bWithGUI && m_bGUI;
+	m_bCMD         = bWithCMD && m_bCMD;
 
 	return( true );
 }
@@ -149,13 +150,18 @@ bool CSG_Tool_Chain::Create(const CSG_Tool_Chain &Tool, bool bWithGUI)
 //---------------------------------------------------------
 bool CSG_Tool_Chain::Create(const CSG_String &File)
 {
-	CSG_MetaData	Chain;
+	if( !SG_File_Cmp_Extension(File, "xml") )
+	{
+		return( false );
+	}
 
 	if( File.Right(sizeof(".pyt.xml")).Make_Lower().Find(".pyt.xml") >= 0 )
 	{
 		return( false );
 	}
 
+	CSG_MetaData Chain;
+
 	if( !Chain.Load(File) )
 	{
 		Error_Fmt("%s: %s", _TL("failed to load or parse xml file"), File.c_str());
@@ -381,7 +387,7 @@ bool CSG_Tool_Chain::Create(const CSG_MetaData &Chain)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -426,7 +432,7 @@ void CSG_Tool_Chain::Add_References(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -454,13 +460,13 @@ int CSG_Tool_Chain::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parame
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CSG_Tool_Chain::On_Execute(void)
 {
-	bool	bResult	= Data_Initialize();
+	bool bResult = Data_Initialize();
 
 	if( !bResult )
 	{
@@ -469,7 +475,7 @@ bool CSG_Tool_Chain::On_Execute(void)
 
 	for(int i=0; bResult && i<m_Chain["tools"].Get_Children_Count(); i++)
 	{
-		bResult	= Tool_Run(m_Chain["tools"][i]);
+		bResult = Tool_Run(m_Chain["tools"][i]);
 	}
 
 	Data_Finalize();
@@ -479,7 +485,7 @@ bool CSG_Tool_Chain::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -490,7 +496,7 @@ bool CSG_Tool_Chain::Data_Add(const CSG_String &ID, CSG_Parameter *pData)
 		return( false );
 	}
 
-	CSG_Parameter	*pParameter	= m_Data(ID);	// check if the local data manager already has a parameter with this identifier
+	CSG_Parameter *pParameter = m_Data(ID);	// check if the local data manager already has a parameter with this identifier
 
 	if( pParameter )	// it has, so don't add twice!
 	{
@@ -592,7 +598,7 @@ bool CSG_Tool_Chain::Data_Add_TempList(const CSG_String &ID, const CSG_String &T
 //---------------------------------------------------------
 bool CSG_Tool_Chain::Data_Del_Temp(const CSG_String &ID, bool bData)
 {
-	CSG_Parameter	*pData	= m_Data(ID);
+	CSG_Parameter *pData = m_Data(ID);
 
 	if( pData )
 	{
@@ -617,9 +623,22 @@ bool CSG_Tool_Chain::Data_Del_Temp(const CSG_String &ID, bool bData)
 	return( true );
 }
 
+//---------------------------------------------------------
+bool CSG_Tool_Chain::Data_Update(const CSG_String &ID, bool bShow)
+{
+	CSG_Parameter *pData = m_Data(ID);
+
+	if( pData && pData->asDataObject() && pData->asDataObject()->is_Valid() )
+	{
+		SG_UI_DataObject_Add(pData->asDataObject(), bShow ? SG_UI_DATAOBJECT_SHOW_MAP : SG_UI_DATAOBJECT_UPDATE);
+	}
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -656,11 +675,11 @@ bool CSG_Tool_Chain::Data_Initialize(void)
 
 	for(int i=0; i<Parameters.Get_Count(); i++)
 	{
-		CSG_Parameter	*pParameter	= Parameters(i);
+		CSG_Parameter &P = Parameters[i];
 
-		if( !(pParameter->is_DataObject() && !pParameter->asDataObject()) )
+		if( !(P.is_DataObject() && !P.asDataObject()) )
 		{
-			if( !Data_Add(pParameter->Get_Identifier(), pParameter) )
+			if( !Data_Add(P.Get_Identifier(), &P) )
 			{
 				return( false );
 			}
@@ -673,30 +692,31 @@ bool CSG_Tool_Chain::Data_Initialize(void)
 //---------------------------------------------------------
 bool CSG_Tool_Chain::Data_Finalize(void)
 {
-	//-----------------------------------------------------
 	for(int i=0; i<Parameters.Get_Count(); i++)	// detach non temporary data before freeing the local data manager !!!
 	{
-		if( Parameters(i)->is_DataObject() )
+		CSG_Parameter &P = Parameters[i];
+
+		if( P.is_DataObject() )
 		{
-			if( m_Data(Parameters(i)->Get_Identifier()) )
+			if( m_Data(P.Get_Identifier()) )
 			{
-				if( Parameters(i)->Get_Type() == PARAMETER_TYPE_DataObject_Output || (Parameters(i)->is_Output() && !Parameters(i)->is_Optional()) )
+				if( P.Get_Type() == PARAMETER_TYPE_DataObject_Output || (P.is_Output() && !P.is_Optional()) )
 				{
-					Parameters(i)->Set_Value(m_Data(Parameters(i)->Get_Identifier())->asDataObject());
+					P.Set_Value(m_Data(P.Get_Identifier())->asDataObject());
 				}
 			}
 
-			m_Data_Manager.Delete(Parameters(i)->asDataObject(), true);
+			m_Data_Manager.Delete(P.asDataObject(), true);
 		}
-		else if( Parameters(i)->is_DataObject_List() )
+		else if( P.is_DataObject_List() )
 		{
-			if( Parameters(i)->is_Output() && m_Data(Parameters(i)->Get_Identifier()) )	// output lists cannot be up-to-date yet
+			if( P.is_Output() && m_Data(P.Get_Identifier()) )	// output lists cannot be up-to-date yet
 			{
-				CSG_Parameter	*pData	= m_Data(Parameters(i)->Get_Identifier());
+				CSG_Parameter *pData = m_Data(P.Get_Identifier());
 
 				for(int j=0; j<Get_List_Count(pData); j++)	// csg_parameter::assign() will not work, if parameters data manager is the standard data manager because it checks for existing data sets
 				{
-					Parameters(i)->asList()->Add_Item(Get_List_Item(pData, j));
+					P.asList()->Add_Item(Get_List_Item(pData, j));
 				}
 			}
 
@@ -707,7 +727,7 @@ bool CSG_Tool_Chain::Data_Finalize(void)
 		}
 	}
 
-	m_Data_Manager.Delete_All();
+	m_Data_Manager.Delete();
 
 	m_Data.Destroy();
 
@@ -762,7 +782,7 @@ bool CSG_Tool_Chain::Data_Finalize(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -790,14 +810,34 @@ bool CSG_Tool_Chain::Parameter_isCompatible(TSG_Parameter_Type A, TSG_Parameter_
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CSG_Tool_Chain::Message(const CSG_MetaData &Message)
+{
+	if( Message.Cmp_Name("message") )
+	{
+		if( IS_TRUE_PROPERTY(Message, "dialog") )
+		{
+			Message_Dlg(Message.Get_Content());
+		}
+
+		Message_Fmt("\n%s", Message.Get_Content().c_str());
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Parameters *pData)
 {
-	//-----------------------------------------------------
-	CSG_String	Type;
+	CSG_String Type;
 
 	if( !Condition.Cmp_Name("condition") || !Condition.Get_Property("type", Type) )
 	{
@@ -805,11 +845,11 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	}
 
 	//-----------------------------------------------------
-	CSG_String	Variable;
+	CSG_String Variable;
 
 	if( !Condition.Get_Property("varname", Variable) && !Condition.Get_Property("variable", Variable) )
 	{
-		Variable	= Condition.Get_Content();
+		Variable = Condition.Get_Content();
 	}
 
 	//-----------------------------------------------------
@@ -821,7 +861,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	//-----------------------------------------------------
 	if( !Type.CmpNoCase("exists"    ) )	// data object exists
 	{
-		CSG_Parameter	*pParameter	= (*pData)(Variable);
+		CSG_Parameter *pParameter = (*pData)(Variable);
 
 		return( pParameter && ((pParameter->is_DataObject() && pParameter->asDataObject()) || (pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count())) );
 	}
@@ -832,7 +872,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	}
 
 	//-----------------------------------------------------
-	CSG_Parameter	*pOption	= (*pData)(Variable);
+	CSG_Parameter *pOption = (*pData)(Variable);
 
 	if( pOption == NULL )
 	{
@@ -844,7 +884,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	//-----------------------------------------------------
 	case PARAMETER_TYPE_Bool     :
 		{
-			CSG_String	Value;
+			CSG_String Value;
 
 			if( Condition.Get_Property("value", Value) )
 			{
@@ -860,7 +900,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	case PARAMETER_TYPE_Data_Type:
 	case PARAMETER_TYPE_Choice   :
 		{
-			int		Value;
+			int Value;
 
 			if( Condition.Get_Property("value", Value) )
 			{
@@ -876,7 +916,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	case PARAMETER_TYPE_Double   :
 	case PARAMETER_TYPE_Degree   :
 		{
-			double	Value;
+			double Value;
 
 			if( Condition.Get_Property("value", Value) )
 			{
@@ -893,7 +933,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	case PARAMETER_TYPE_Text     :
 	case PARAMETER_TYPE_FilePath :
 		{
-			CSG_String	Value;
+			CSG_String Value;
 
 			Condition.Get_Property("value", Value);	// no 'if', bcos empty string would return false !!
 
@@ -909,7 +949,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	//-----------------------------------------------------
 	case PARAMETER_TYPE_Grid_System:
 		{
-			CSG_String	Value;
+			CSG_String Value;
 
 			if( Condition.Get_Property("value", Value) )
 			{
@@ -932,7 +972,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -947,9 +987,9 @@ bool CSG_Tool_Chain::ForEach(const CSG_MetaData &Commands)
 	}
 
 	//-----------------------------------------------------
-	bool	bIgnoreErrors	= IS_TRUE_PROPERTY(Commands, "ignore_errors");
+	bool bIgnoreErrors = IS_TRUE_PROPERTY(Commands, "ignore_errors");
 
-	CSG_String	VarName;
+	CSG_String VarName;
 
 	if( Commands.Get_Property("varname", VarName) )
 	{
@@ -959,7 +999,7 @@ bool CSG_Tool_Chain::ForEach(const CSG_MetaData &Commands)
 	if( Commands.Get_Property("input", VarName) )
 	{
 		return( ForEach_Object  (Commands, VarName, bIgnoreErrors)
-			||  ForEach_File    (Commands, VarName, bIgnoreErrors) );
+		    ||  ForEach_File    (Commands, VarName, bIgnoreErrors) );
 	}
 
 	Error_Set("foreach statement misses iterator or input list definition");
@@ -970,7 +1010,7 @@ bool CSG_Tool_Chain::ForEach(const CSG_MetaData &Commands)
 //---------------------------------------------------------
 bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_String &VarName, bool bIgnoreErrors)
 {
-	CSG_Parameter	*pIterator	= Parameters(VarName);
+	CSG_Parameter *pIterator = Parameters(VarName);
 
 	if( pIterator )
 	{
@@ -980,10 +1020,10 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 	}
 
 	//-----------------------------------------------------
-	CSG_String	s;
+	CSG_String s;
 
-	double	begin	= Commands.Get_Property("begin", s) ? (Parameters(s) ? Parameters[s].asDouble() : s.asDouble()) : 0.;
-	double	end		= Commands.Get_Property("end"  , s) ? (Parameters(s) ? Parameters[s].asDouble() : s.asDouble()) : 0.;
+	double begin = Commands.Get_Property("begin", s) ? (Parameters(s) ? Parameters[s].asDouble() : s.asDouble()) : 0.;
+	double   end = Commands.Get_Property("end"  , s) ? (Parameters(s) ? Parameters[s].asDouble() : s.asDouble()) : 0.;
 
 	if( begin >= end )
 	{
@@ -992,20 +1032,20 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 		return( false );
 	}
 
-	double	step	= 1.;
+	double step = 1.;
 
 	if( Commands.Get_Property("steps", s) )
 	{
-		double	steps	= Parameters(s) ? Parameters[s].asDouble() : s.asDouble();
+		double steps = Parameters(s) ? Parameters[s].asDouble() : s.asDouble();
 
 		if( steps > 0 )
 		{
-			step	= (end - begin) / steps;
+			step = (end - begin) / steps;
 		}
 	}
 	else if( Commands.Get_Property("step", s) )
 	{
-		 step	= Parameters(s) ? Parameters[s].asDouble() : s.asDouble();
+		 step = Parameters(s) ? Parameters[s].asDouble() : s.asDouble();
 	}
 
 	if( step <= 0. )
@@ -1018,9 +1058,9 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 	Message_Fmt("\nfor i = %f to %f step %f (%d steps)", begin, end, step, (int)((end - begin) / step));
 
 	//-----------------------------------------------------
-	bool	bResult	= true;
+	bool bResult = true;
 
-	pIterator	= Parameters.Add_Double("", VarName, "Iterator", "");
+	pIterator = Parameters.Add_Double("", VarName, "Iterator", "");
 
 	for(double i=begin; bResult && i<=end; i+=step)
 	{
@@ -1030,7 +1070,7 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 
 		for(int iTool=0; bResult && iTool<Commands.Get_Children_Count(); iTool++)
 		{
-			const CSG_MetaData	&Tool	= Commands[iTool];
+			const CSG_MetaData &Tool = Commands[iTool];
 
 			if( Tool.Cmp_Name("tool") )
 			{
@@ -1038,7 +1078,7 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 
 				if( !bResult && bIgnoreErrors )
 				{
-					bResult	= true;
+					bResult = true;
 				}
 			}
 		}
@@ -1052,7 +1092,7 @@ bool CSG_Tool_Chain::ForEach_Iterator(const CSG_MetaData &Commands, const CSG_St
 //---------------------------------------------------------
 bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_String &ListVarName, bool bIgnoreErrors)
 {
-	CSG_Parameter	*pList	= m_Data(ListVarName);
+	CSG_Parameter *pList = m_Data(ListVarName);
 
 	if( !pList )
 	{
@@ -1060,7 +1100,7 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 	}
 
 	//-----------------------------------------------------
-	bool	bResult	= true;
+	bool bResult = true;
 
 	if( pList->is_DataObject_List() )
 	{
@@ -1068,7 +1108,7 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 		{
 			for(int iTool=0; bResult && iTool<Commands.Get_Children_Count(); iTool++)
 			{
-				const CSG_MetaData	&Tool	= Commands[iTool];
+				const CSG_MetaData &Tool = Commands[iTool];
 
 				if( Tool.Cmp_Name("tool") )
 				{
@@ -1081,12 +1121,12 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 					}
 				}
 
-				bResult	= Tool_Run(Tool, bIgnoreErrors);
+				bResult = Tool_Run(Tool, bIgnoreErrors);
 			}
 
 			if( !bResult && bIgnoreErrors )
 			{
-				bResult	= true;
+				bResult = true;
 			}
 		}
 	}
@@ -1098,7 +1138,7 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 		{
 			for(int iTool=0; bResult && iTool<Commands.Get_Children_Count(); iTool++)
 			{
-				const CSG_MetaData	&Tool	= Commands[iTool];
+				const CSG_MetaData &Tool = Commands[iTool];
 
 				if( Tool.Cmp_Name("tool") )
 				{
@@ -1111,12 +1151,12 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 					}
 				}
 
-				bResult	= Tool_Run(Tool, bIgnoreErrors);
+				bResult = Tool_Run(Tool, bIgnoreErrors);
 			}
 
 			if( !bResult && bIgnoreErrors )
 			{
-				bResult	= true;
+				bResult = true;
 			}
 		}
 	}
@@ -1127,29 +1167,29 @@ bool CSG_Tool_Chain::ForEach_Object(const CSG_MetaData &Commands, const CSG_Stri
 //---------------------------------------------------------
 bool CSG_Tool_Chain::ForEach_File(const CSG_MetaData &Commands, const CSG_String &ListVarName, bool bIgnoreErrors)
 {
-	CSG_Parameter	*pList	= Parameters(ListVarName);
+	CSG_Parameter *pList = Parameters(ListVarName);
 
 	if( !pList || pList->Get_Type() != PARAMETER_TYPE_FilePath )
 	{
 		return( false );
 	}
 
-	CSG_Strings	Files;
+	CSG_Strings Files;
 
 	pList->asFilePath()->Get_FilePaths(Files);
 
 	//-----------------------------------------------------
-	bool	bResult	= true;
+	bool bResult = true;
 
 	for(int iFile=0; bResult && iFile<Files.Get_Count(); iFile++)
 	{
 		for(int iTool=0; bResult && iTool<Commands.Get_Children_Count(); iTool++)
 		{
-			const CSG_MetaData	&Tool	= Commands[iTool];
+			const CSG_MetaData &Tool = Commands[iTool];
 
 			if( Tool.Cmp_Name("tool") )
 			{
-				CSG_Array_Int	Input;
+				CSG_Array_Int Input;
 
 				for(int j=0; j<Tool.Get_Children_Count(); j++)
 				{
@@ -1158,11 +1198,11 @@ bool CSG_Tool_Chain::ForEach_File(const CSG_MetaData &Commands, const CSG_String
 						Tool(j)->Set_Content(Files[iFile]);
 						Tool(j)->Set_Property("varname", "false");
 
-						Input	+= j;
+						Input += j;
 					}
 				}
 
-				bResult	= Tool_Run(Tool, bIgnoreErrors);
+				bResult = Tool_Run(Tool, bIgnoreErrors);
 
 				for(size_t i=0; i<Input.Get_uSize(); i++)
 				{
@@ -1172,12 +1212,12 @@ bool CSG_Tool_Chain::ForEach_File(const CSG_MetaData &Commands, const CSG_String
 			}
 			else
 			{
-				bResult	= Tool_Run(Tool, bIgnoreErrors);
+				bResult = Tool_Run(Tool, bIgnoreErrors);
 			}
 
 			if( !bResult && bIgnoreErrors )
 			{
-				bResult	= true;
+				bResult = true;
 			}
 		}
 	}
@@ -1187,12 +1227,19 @@ bool CSG_Tool_Chain::ForEach_File(const CSG_MetaData &Commands, const CSG_String
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 {
+	if( Tool.Cmp_Name("message") )
+	{
+		Message(Tool);
+
+		return( true );
+	}
+
 	//-----------------------------------------------------
 	if( Tool.Cmp_Name("comment") )
 	{
@@ -1210,10 +1257,15 @@ bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 		return( Data_Del_Temp(Tool.Get_Content(), IS_TRUE_PROPERTY(Tool, "data")) );
 	}
 
+	if( Tool.Cmp_Name("update") )
+	{
+		return( Data_Update(Tool.Get_Content(), IS_TRUE_PROPERTY(Tool, "show")) );
+	}
+
 	//-----------------------------------------------------
 	if( Tool.Cmp_Name("condition") )
 	{
-		const CSG_MetaData	*pTools	= (!Check_Condition(Tool, &m_Data) || !Check_Condition(Tool, &Parameters))
+		const CSG_MetaData *pTools = (!Check_Condition(Tool, &m_Data) || !Check_Condition(Tool, &Parameters))
 			? Tool("else") : (Tool("if") ? Tool("if") : &Tool);
 
 		for(int i=0; pTools && i<pTools->Get_Children_Count(); i++)
@@ -1247,9 +1299,9 @@ bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 	}
 
 	//-----------------------------------------------------
-	const SG_Char *Name	= Tool.Get_Property("tool") ? Tool.Get_Property("tool") : Tool.Get_Property("module");
+	const SG_Char *Name = Tool.Get_Property("tool") ? Tool.Get_Property("tool") : Tool.Get_Property("module");
 
-	CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Create_Tool(Tool.Get_Property("library"), Name,
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool(Tool.Get_Property("library"), Name,
 		IS_TRUE_PROPERTY(Tool, "with_gui")	// this option allows to run a tool in 'gui-mode', e.g. to popup variogram dialogs for kriging interpolation
 	);
 
@@ -1296,7 +1348,7 @@ bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1318,7 +1370,7 @@ bool CSG_Tool_Chain::Tool_Check_Condition(const CSG_MetaData &Tool)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1400,6 +1452,13 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 	{
 		const CSG_MetaData &Parameter = Tool[i]; if( Parameter.Cmp_Name("comment") ) { continue; }
 
+		if( Parameter.Cmp_Name("message") )
+		{
+			Message(Parameter);
+
+			continue;
+		}
+
 		CSG_Parameter *pParameter, *pOwner;
 
 		if( !Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner) )
@@ -1431,6 +1490,18 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 
 		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
 
+		if( pParameter->Get_Type() == PARAMETER_TYPE_Grid_System )
+		{
+			CSG_Parameter *pData = m_Data(Parameter.Get_Content());
+
+			if( pData && pData->asDataObject() && pData->asDataObject()->asGrid() && pData->asDataObject()->asGrid()->Get_System().is_Valid() )
+			{
+				pParameter->asGrid_System()->Create(pData->asDataObject()->asGrid()->Get_System());
+
+				continue;
+			}
+		}
+
 		if( IS_TRUE_PROPERTY(Parameter, "varname")
 			? pParameter->Set_Value(Parameters(Parameter.Get_Content()))
 			: pParameter->Set_Value(           Parameter.Get_Content() ) )
@@ -1663,15 +1734,15 @@ bool CSG_Tool_Chain::Tool_Finalize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 CSG_String CSG_Tool_Chain::Get_Script(CSG_Tool *pTool, bool bHeader, bool bAllParameters)
 {
-	CSG_MetaData	Tool;	Tool.Set_Name("tool");
+	CSG_MetaData Tool; Tool.Set_Name("tool");
 
 	Tool.Add_Property("library", pTool->Get_Library());
 	Tool.Add_Property("tool"   , pTool->Get_ID     ());
@@ -1690,7 +1761,7 @@ CSG_String CSG_Tool_Chain::Get_Script(CSG_Tool *pTool, bool bHeader, bool bAllPa
 	}
 
 	//-----------------------------------------------------
-	CSG_MetaData	Parameters;
+	CSG_MetaData Parameters;
 
 	_Get_Script_Parameters(Parameters, pTool->Get_Parameters(), "");
 
@@ -1700,7 +1771,7 @@ CSG_String CSG_Tool_Chain::Get_Script(CSG_Tool *pTool, bool bHeader, bool bAllPa
 	}
 
 	//-----------------------------------------------------
-	CSG_MetaData	Tools;	Tools.Set_Name("toolchain");
+	CSG_MetaData Tools; Tools.Set_Name("toolchain");
 
 	Tools.Add_Property("saga-version", SAGA_VERSION);
 
@@ -1723,23 +1794,23 @@ bool CSG_Tool_Chain::_Get_Script_Tool(CSG_MetaData &Tool, CSG_Parameters *pParam
 {
 	for(int iParameter=0; iParameter<pParameters->Get_Count(); iParameter++)
 	{
-		CSG_Parameter	*pParameter	= pParameters->Get_Parameter(iParameter);
+		CSG_Parameter &P = *pParameters->Get_Parameter(iParameter);
 
-		if( !bAllParameters && (!pParameter->is_Enabled(false) || pParameter->is_Information()) )
+		if( !bAllParameters && (!P.is_Enabled(false) || P.is_Information()) )
 		{
 			continue;
 		}
 
-		CSG_MetaData	*pChild	= NULL;
+		CSG_MetaData *pChild = NULL;
 
-		switch( pParameter->Get_Type() )
+		switch( P.Get_Type() )
 		{
 		case PARAMETER_TYPE_Parameters  :
-			_Get_Script_Tool(Tool, pParameter->asParameters(), bAllParameters, Prefix + pParameter->Get_Identifier() + '.', true);
+			_Get_Script_Tool(Tool, P.asParameters(), bAllParameters, Prefix + P.Get_Identifier() + '.', true);
 			break;
 
 		case PARAMETER_TYPE_Bool        :
-			pChild	= Tool.Add_Child("option", pParameter->asBool() ? "true" : "false");
+			pChild	= Tool.Add_Child("option", P.asBool() ? "true" : "false");
 			break;
 
 		case PARAMETER_TYPE_Int         :
@@ -1753,33 +1824,33 @@ bool CSG_Tool_Chain::_Get_Script_Tool(CSG_MetaData &Tool, CSG_Parameters *pParam
 		case PARAMETER_TYPE_Choices     :
 		case PARAMETER_TYPE_Table_Field :
 		case PARAMETER_TYPE_Table_Fields:
-			pChild	= Tool.Add_Child("option", pParameter->asString());
+			pChild	= Tool.Add_Child("option", P.asString());
 			break;
 
 		case PARAMETER_TYPE_Data_Type   :
 		case PARAMETER_TYPE_Choice      :
-			pChild	= Tool.Add_Child("option", pParameter->asInt());
+			pChild	= Tool.Add_Child("option", P.asInt());
 			break;
 
 		case PARAMETER_TYPE_FixedTable  :
 			pChild	= Tool.Add_Child("option");
-			pParameter->Serialize(*pChild, true);
+			P.Serialize(*pChild, true);
 			break;
 
 		case PARAMETER_TYPE_Grid_System :
-			if( pParameter->Get_Children_Count() == 0 )
+			if( P.Get_Children_Count() == 0 )
 			{
-				pChild	= Tool.Add_Child("option", pParameter->asString());
+				pChild	= Tool.Add_Child("option", P.asString());
 			}
 			break;
 
 		default:
-			if( pParameter->is_Input() )
+			if( P.is_Input() )
 			{
-				pChild	= Tool.Add_Child("input");
-				pChild->Set_Content(pParameter->is_Optional() ? "INPUT_OPTIONAL" : "INPUT");
+				pChild 	= Tool.Add_Child("input");
+				pChild->Set_Content(P.is_Optional() ? "INPUT_OPTIONAL" : "INPUT");
 			}
-			else if( pParameter->is_Output() )
+			else if( P.is_Output() )
 			{
 				pChild	= Tool.Add_Child("output");
 				pChild->Set_Content("OUTPUT");
@@ -1789,16 +1860,16 @@ bool CSG_Tool_Chain::_Get_Script_Tool(CSG_MetaData &Tool, CSG_Parameters *pParam
 
 		if( pChild )
 		{
-			pChild->Add_Property("id", Prefix + pParameter->Get_Identifier());
+			pChild->Add_Property("id", Prefix + P.Get_Identifier());
 
 			if( bVarNames )
 			{
-				if( pParameter->is_Option() )
+				if( P.is_Option() )
 				{
 					pChild->Add_Property("varname", "true");
 				}
 
-				pChild->Set_Content(Prefix + pParameter->Get_Identifier());
+				pChild->Set_Content(Prefix + P.Get_Identifier());
 			}
 		}
 	}
@@ -1811,77 +1882,77 @@ bool CSG_Tool_Chain::_Get_Script_Parameters(CSG_MetaData &Parameters, CSG_Parame
 {
 	for(int i=0; i<pParameters->Get_Count(); i++)
 	{
-		CSG_Parameter	*pParameter	= pParameters->Get_Parameter(i);
+		CSG_Parameter &P = *pParameters->Get_Parameter(i);
 
-		if( pParameter->Get_Type() == PARAMETER_TYPE_Parameters )
+		if( P.Get_Type() == PARAMETER_TYPE_Parameters )
 		{
-			_Get_Script_Parameters(Parameters, pParameter->asParameters(), Prefix + pParameter->Get_Identifier() + '.');
+			_Get_Script_Parameters(Parameters, P.asParameters(), Prefix + P.Get_Identifier() + '.');
 
 			continue;	// no support for sub-parameter-lists in here
 		}
 
-		CSG_MetaData	&Parameter	= *Parameters.Add_Child(
-			pParameter->is_Option() ? "option" :
-			pParameter->is_Output() ? "output" : "input"
+		CSG_MetaData &Parameter = *Parameters.Add_Child(
+			P.is_Option() ? "option" :
+			P.is_Output() ? "output" : "input"
 		);
 
-		Parameter.Add_Property("varname"    , pParameter->Get_Identifier ());
-		Parameter.Add_Property("type"       , SG_Parameter_Type_Get_Identifier(pParameter->Get_Type()));
-		Parameter.Add_Child   ("name"       , pParameter->Get_Name       ());
-		Parameter.Add_Child   ("description", pParameter->Get_Description());
+		Parameter.Add_Property("varname"    , P.Get_Identifier ());
+		Parameter.Add_Property("type"       , SG_Parameter_Type_Get_Identifier(P.Get_Type()));
+		Parameter.Add_Child   ("name"       , P.Get_Name       ());
+		Parameter.Add_Child   ("description", P.Get_Description());
 
-		if( pParameter->Get_Parent() )
+		if( P.Get_Parent() )
 		{
-			Parameter.Add_Property("parent", pParameter->Get_Parent()->Get_Identifier());
+			Parameter.Add_Property("parent", P.Get_Parent()->Get_Identifier());
 		}
 
-		if( pParameter->Get_Type() == PARAMETER_TYPE_Node
-		||  pParameter->Get_Type() == PARAMETER_TYPE_Grid_System )
+		if( P.Get_Type() == PARAMETER_TYPE_Node
+		||  P.Get_Type() == PARAMETER_TYPE_Grid_System )
 		{
 			continue;	// nothing more to do for these types
 		}
 
-		if( pParameter->is_Option() )
+		if( P.is_Option() )
 		{
-			CSG_MetaData	&Value	= *Parameter.Add_Child("value", pParameter->asString());
+			CSG_MetaData &Value = *Parameter.Add_Child("value", P.asString());
 
-			if( pParameter->asValue() )
+			if( P.asValue() )
 			{
-				if( pParameter->asValue()->has_Minimum() ) Value.Add_Property("min", pParameter->asValue()->Get_Minimum());
-				if( pParameter->asValue()->has_Maximum() ) Value.Add_Property("max", pParameter->asValue()->Get_Maximum());
+				if( P.asValue()->has_Minimum() ) Value.Add_Property("min", P.asValue()->Get_Minimum());
+				if( P.asValue()->has_Maximum() ) Value.Add_Property("max", P.asValue()->Get_Maximum());
 			}
 
-			if( pParameter->asChoice () ) Parameter.Add_Child("choices", pParameter->asChoice ()->Get_Items());
-			if( pParameter->asChoices() ) Parameter.Add_Child("choices", pParameter->asChoices()->Get_Items());
+			if( P.asChoice () ) Parameter.Add_Child("choices", P.asChoice ()->Get_Items());
+			if( P.asChoices() ) Parameter.Add_Child("choices", P.asChoices()->Get_Items());
 
-			if( pParameter->asFilePath() )
+			if( P.asFilePath() )
 			{
-				Parameter.Add_Property("save"     , pParameter->asFilePath()->is_Save     () ? "true" : "false");
-				Parameter.Add_Property("directory", pParameter->asFilePath()->is_Directory() ? "true" : "false");
-				Parameter.Add_Property("multiple" , pParameter->asFilePath()->is_Multiple () ? "true" : "false");
-				Parameter.Add_Child   ("filter"   , pParameter->asFilePath()->Get_Filter());
+				Parameter.Add_Property("save"     , P.asFilePath()->is_Save     () ? "true" : "false");
+				Parameter.Add_Property("directory", P.asFilePath()->is_Directory() ? "true" : "false");
+				Parameter.Add_Property("multiple" , P.asFilePath()->is_Multiple () ? "true" : "false");
+				Parameter.Add_Child   ("filter"   , P.asFilePath()->Get_Filter());
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_FixedTable )
+			if( P.Get_Type() == PARAMETER_TYPE_FixedTable )
 			{
-				pParameter->Serialize(Parameter, true);
+				P.Serialize(Parameter, true);
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_Table_Field )
+			if( P.Get_Type() == PARAMETER_TYPE_Table_Field )
 			{
-				Value.Set_Content(pParameter->is_Optional() ? "true" : "false");
+				Value.Set_Content(P.is_Optional() ? "true" : "false");
 			}
 		}
 		else	// data objects
 		{
-			if( pParameter->is_Optional() )
+			if( P.is_Optional() )
 			{
 				Parameter.Add_Property("optional", "true");
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_DataObject_Output )
+			if( P.Get_Type() == PARAMETER_TYPE_DataObject_Output )
 			{
-				switch( ((CSG_Parameter_Data_Object_Output *)pParameter)->Get_DataObject_Type() )
+				switch( ((CSG_Parameter_Data_Object_Output *)&P)->Get_DataObject_Type() )
 				{
 				case SG_DATAOBJECT_TYPE_Grid:
 					Parameter.Set_Property("type"  , SG_Parameter_Type_Get_Identifier(PARAMETER_TYPE_Grid));
@@ -1898,9 +1969,9 @@ bool CSG_Tool_Chain::_Get_Script_Parameters(CSG_MetaData &Parameters, CSG_Parame
 				}
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_Shapes )
+			if( P.Get_Type() == PARAMETER_TYPE_Shapes )
 			{
-				switch( ((CSG_Parameter_Shapes *)pParameter)->Get_Shape_Type() )
+				switch( ((CSG_Parameter_Shapes *)&P)->Get_Shape_Type() )
 				{
 				case SHAPE_TYPE_Point  : Parameter.Add_Property("feature_type", "point"  ); break;
 				case SHAPE_TYPE_Points : Parameter.Add_Property("feature_type", "points" ); break;
@@ -1910,14 +1981,14 @@ bool CSG_Tool_Chain::_Get_Script_Parameters(CSG_MetaData &Parameters, CSG_Parame
 				}
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_Grid_List
-				&&  !((CSG_Parameter_Grid_List *)pParameter)->Get_System() )
+			if( P.Get_Type() == PARAMETER_TYPE_Grid_List
+			&&  !((CSG_Parameter_Grid_List *)&P)->Get_System() )
 			{
 				Parameter.Add_Property("no_system", "true");
 			}
 
-			if( pParameter->Get_Type() == PARAMETER_TYPE_Grids_List
-				&&  !((CSG_Parameter_Grid_List *)pParameter)->Get_System() )
+			if( P.Get_Type() == PARAMETER_TYPE_Grids_List
+			&&  !((CSG_Parameter_Grid_List *)&P)->Get_System() )
 			{
 				Parameter.Add_Property("no_system", "true");
 			}
@@ -1929,9 +2000,9 @@ bool CSG_Tool_Chain::_Get_Script_Parameters(CSG_MetaData &Parameters, CSG_Parame
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2011,7 +2082,7 @@ CSG_Tool_Chains::~CSG_Tool_Chains(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2041,24 +2112,24 @@ bool CSG_Tool_Chains::Add_Tool(CSG_Tool_Chain *pTool)
 //---------------------------------------------------------
 CSG_Tool * CSG_Tool_Chains::Get_Tool(int Index, TSG_Tool_Type Type) const
 {
-	CSG_Tool	*pTool	= Index >= 0 && Index < Get_Count() ? (CSG_Tool_Chain *)m_Tools[Index] : NULL;
+	CSG_Tool *pTool = Index >= 0 && Index < Get_Count() ? (CSG_Tool_Chain *)m_Tools[Index] : NULL;
 
 	return(	pTool && (Type == TOOL_TYPE_Base || Type == pTool->Get_Type()) ? pTool : NULL );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Tool * CSG_Tool_Chains::Create_Tool(const CSG_String &Name, bool bWithGUI)
+CSG_Tool * CSG_Tool_Chains::Create_Tool(const CSG_String &Name, bool bWithGUI, bool bWithCMD)
 {
-	CSG_Tool	*pTool	= CSG_Tool_Library::Get_Tool(Name);
+	CSG_Tool *pTool = CSG_Tool_Library::Get_Tool(Name);
 
 	if( pTool && pTool->Get_Type() == TOOL_TYPE_Chain )
 	{
-		m_xTools.Add(pTool = new CSG_Tool_Chain(*((CSG_Tool_Chain *)pTool), bWithGUI));
+		m_xTools.Add(pTool = new CSG_Tool_Chain(*((CSG_Tool_Chain *)pTool), bWithGUI, bWithCMD));
 
 		return( pTool );
 	}
@@ -2094,9 +2165,9 @@ bool CSG_Tool_Chains::Delete_Tools(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2274,9 +2345,9 @@ bool CSG_Tool_Chain::_Save_History_Add_Input(const CSG_MetaData &History, CSG_Me
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
