@@ -226,7 +226,8 @@ CSG_3DView_Panel::CSG_3DView_Panel(wxWindow *pParent, CSG_Grid *pDrape)
 	m_Parameters.Add_Choice("3D_VIEW" , "NORTH"       , _TL("North Arrow"          ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("no"), _TL("yes"), _TL("yes, without bounding box")), m_North);
 	m_Parameters.Add_Double("NORTH"   , "NORTH_SIZE"  , _TL("Size"                 ), _TL(""), m_North_Size, 1., true);
 
-	m_Parameters.Add_Choice("3D_VIEW" , "LABELS"      , _TL("Axis Labeling"        ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("all"), _TL("horizontal"), _TL("none")), m_Labels);
+	m_Parameters.Add_Choice("3D_VIEW" , "LABELS"      , _TL("Axis Labeling"        ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("all axes"), _TL("frontal axes"), _TL("none")), m_Labels);
+	m_Parameters.Add_Choice("LABELS"  , "LABEL_DIR"   , _TL("Label Orientation"    ), _TL(""), CSG_String::Format("%s|%s", _TL("along axis"), _TL("cross axis")), m_Label_Dir);
 	m_Parameters.Add_Int   ("LABELS"  , "LABEL_RES"   , _TL("Resolution"           ), _TL(""), m_Label_Res, 20, true, 1000, true);
 	m_Parameters.Add_Double("LABELS"  , "LABEL_SCALE" , _TL("Size"                 ), _TL(""), m_Label_Scale, 0.1, true, 10., true);
 
@@ -240,14 +241,7 @@ CSG_3DView_Panel::CSG_3DView_Panel(wxWindow *pParent, CSG_Grid *pDrape)
 	{
 		m_Parameters.Add_Node("", "MAP", _TL("Map Draping"), _TL(""));
 		m_Parameters.Add_Bool("MAP", "MAP_DRAPE", _TL("Map Draping"), _TL(""), true);
-		m_Parameters.Add_Choice("MAP", "MAP_DRAPE_MODE", _TL("Map Drape Resampling"), _TL(""),
-			CSG_String::Format("%s|%s|%s|%s",
-				_TL("Nearest Neighbour"),
-				_TL("Bilinear Interpolation"),
-				_TL("Bicubic Spline Interpolation"),
-				_TL("B-Spline Interpolation")
-			), 0
-		);
+		m_Parameters.Add_Grid_Resampling("MAP", "MAP_DRAPE_MODE", _TL("Map Drape Resampling"), _TL(""), CSG_Grid_Resampling::Interpolations, CSG_Grid_Resampling::NearestNeighbour);
 	}
 
 	//-----------------------------------------------------
@@ -354,6 +348,7 @@ bool CSG_3DView_Panel::Update_Parameters(bool bSave)
 		m_Parameters["NORTH_SIZE"  ].Set_Value(m_North_Size );
 
 		m_Parameters["LABELS"      ].Set_Value(m_Labels     );
+		m_Parameters["LABEL_DIR"   ].Set_Value(m_Label_Dir  );
 		m_Parameters["LABEL_RES"   ].Set_Value(m_Label_Res  );
 		m_Parameters["LABEL_SCALE" ].Set_Value(m_Label_Scale);
 
@@ -394,15 +389,16 @@ bool CSG_3DView_Panel::Update_Parameters(bool bSave)
 		m_North_Size  = m_Parameters["NORTH_SIZE" ].asDouble();
 
 		m_Labels      = m_Parameters["LABELS"     ].asInt   ();
+		m_Label_Dir   = m_Parameters["LABEL_DIR"  ].asInt   ();
 		m_Label_Res   = m_Parameters["LABEL_RES"  ].asInt   ();
 		m_Label_Scale = m_Parameters["LABEL_SCALE"].asDouble();
 
 		switch( m_Parameters("MAP_DRAPE_MODE") ? m_Parameters["MAP_DRAPE_MODE"].asInt() : 0 )
 		{
-		default: m_Drape_Mode = GRID_RESAMPLING_NearestNeighbour; break;
-		case  1: m_Drape_Mode = GRID_RESAMPLING_Bilinear        ; break;
-		case  2: m_Drape_Mode = GRID_RESAMPLING_BicubicSpline   ; break;
-		case  3: m_Drape_Mode = GRID_RESAMPLING_BSpline         ; break;
+		default: m_Drape_Mode = CSG_Grid_Resampling::NearestNeighbour; break;
+		case  1: m_Drape_Mode = CSG_Grid_Resampling::Bilinear        ; break;
+		case  2: m_Drape_Mode = CSG_Grid_Resampling::Bicubic_1   ; break;
+		case  3: m_Drape_Mode = CSG_Grid_Resampling::Bicubic_2         ; break;
 		}
 	}
 
@@ -1013,7 +1009,7 @@ bool CSG_3DView_Panel::Update_View(bool bStatistics)
 	}
 
 	//-----------------------------------------------------
-	CSG_Grid *pDrape = m_pDrape; if( m_pDrape && !m_Parameters("MAP_DRAPE")->asBool() ) m_pDrape = NULL;
+	CSG_Grid *pDrape = m_pDrape; if( m_pDrape && (!m_pDrape->is_Valid() || !m_Parameters("MAP_DRAPE")->asBool()) ) { m_pDrape = NULL; }
 
 	if( Draw() )
 	{
@@ -1346,6 +1342,68 @@ bool CSG_3DView_Panel::_Play(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+//														 //
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+#include "saga_gdi/sgdi_helper.h"
+
+//---------------------------------------------------------
+BEGIN_EVENT_TABLE(CSG_3DView_Legend, wxPanel)
+	EVT_PAINT(CSG_3DView_Legend::On_Paint)
+END_EVENT_TABLE()
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_3DView_Legend::CSG_3DView_Legend(wxWindow *pParent, CSG_Parameter *pColors, CSG_Parameter *pRange, const wxSize &Size)
+	: wxPanel(pParent, wxID_ANY, wxDefaultPosition, Size, wxNO_FULL_REPAINT_ON_RESIZE)
+{
+	m_pColors = pColors;
+	m_pRange  = pRange;
+}
+
+//---------------------------------------------------------
+void CSG_3DView_Legend::On_Paint(wxPaintEvent &WXUNUSED(event))
+{
+	if( IsShown() )
+	{
+		wxPaintDC dc(this); Draw(dc, GetClientSize());
+	}
+}
+
+//---------------------------------------------------------
+void CSG_3DView_Legend::Draw(wxDC &dc, const wxRect &r)
+{
+	if( IsShown() )
+	{
+		CSG_Colors Colors(*m_pColors->asColors());
+
+		int y = r.GetBottom() - 4 - dc.GetFont().GetPixelSize().y;
+
+		for(int x=0; x<r.GetWidth(); x++)
+		{
+			wxColour Color((unsigned long)Colors.Get_Interpolated(x * (Colors.Get_Count() - 1.) / (double)r.GetWidth()));
+			dc.SetPen(wxPen(Color)); dc.DrawLine(r.GetLeft() + x, y, r.GetLeft() + x, r.GetTop());
+		}
+
+		Draw_Edge(dc, EDGE_STYLE_SUNKEN, r.GetLeft(), r.GetTop(), r.GetRight(), y);
+
+		CSG_String Minimum = SG_Get_String(m_pRange->asRange()->Get_Min(), -6);
+		CSG_String Maximum = SG_Get_String(m_pRange->asRange()->Get_Max(), -6);
+
+		Draw_Text(dc, TEXTALIGN_BOTTOMLEFT , r.GetLeft (), r.GetBottom(), Minimum.c_str());
+		Draw_Text(dc, TEXTALIGN_BOTTOMRIGHT, r.GetRight(), r.GetBottom(), Maximum.c_str());
+	}
+}
+
+
 ///////////////////////////////////////////////////////////
 //                                                       //
 //                                                       //
