@@ -94,9 +94,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -113,7 +113,7 @@ CWKSP_Project::~CWKSP_Project(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -154,7 +154,7 @@ bool CWKSP_Project::_Set_Project_Name(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -198,7 +198,7 @@ bool CWKSP_Project::Save(const wxString &File, bool bSaveModified)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -206,7 +206,7 @@ bool CWKSP_Project::Copy(void)
 {
 	wxString File;
 
-	if( !DLG_Save(File, ID_DLG_PROJECT_SAVE) )
+	if( !DLG_Save(File, _TL("Copy Project to..."), wxString::Format("%s|", _TL("SAGA Project"))) )
 	{
 		return( false );
 	}
@@ -225,43 +225,27 @@ bool CWKSP_Project::Copy(void)
 		Directory.Mkdir(wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
 	}
 
-	for(size_t i=0; i<SG_Get_Data_Manager().Table     ().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Table     ().Get(i)), Directory.GetFullPath());
-	}
-
-	for(size_t i=0; i<SG_Get_Data_Manager().Shapes    ().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Shapes    ().Get(i)), Directory.GetFullPath());
-	}
-
-	for(size_t i=0; i<SG_Get_Data_Manager().PointCloud().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().PointCloud().Get(i)), Directory.GetFullPath());
-	}
-
-	for(size_t i=0; i<SG_Get_Data_Manager().TIN       ().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().TIN       ().Get(i)), Directory.GetFullPath());
-	}
-
-	for(size_t i=0; i<SG_Get_Data_Manager().Grid      ().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Grid      ().Get(i)), Directory.GetFullPath());
-	}
-
-	for(size_t i=0; i<SG_Get_Data_Manager().Grids     ().Count(); i++)
-	{
-		_Copy_To_File(g_pData->Get(SG_Get_Data_Manager().Grids     ().Get(i)), Directory.GetFullPath());
-	}
+	for(size_t i=0; i<SG_Get_Data_Manager().Table     ().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().Table     ().Get(i), Directory.GetFullPath()); }
+	for(size_t i=0; i<SG_Get_Data_Manager().Shapes    ().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().Shapes    ().Get(i), Directory.GetFullPath()); }
+	for(size_t i=0; i<SG_Get_Data_Manager().PointCloud().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().PointCloud().Get(i), Directory.GetFullPath()); }
+	for(size_t i=0; i<SG_Get_Data_Manager().TIN       ().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().TIN       ().Get(i), Directory.GetFullPath()); }
+	for(size_t i=0; i<SG_Get_Data_Manager().Grid      ().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().Grid      ().Get(i), Directory.GetFullPath()); }
+	for(size_t i=0; i<SG_Get_Data_Manager().Grids     ().Count(); i++) { _Copy_To_File(SG_Get_Data_Manager().Grids     ().Get(i), Directory.GetFullPath()); }
 
 	return( _Save(Directory.GetFullPath(), false, true) );
 }
 
 //---------------------------------------------------------
-bool CWKSP_Project::_Copy_To_File(CWKSP_Data_Item *pItem, const wxString &Directory)
+bool CWKSP_Project::_Copy_To_File(CSG_Data_Object *pObject, const wxString &Directory)
 {
-	wxFileName	Path(Directory);
+	CWKSP_Data_Item *pItem = g_pData->Get(pObject);
+
+	if( !pItem )
+	{
+		return( false );
+	}
+
+	wxFileName Path(Directory);
 
 	switch( pItem->Get_Type() )	// default extensions
 	{
@@ -269,13 +253,13 @@ bool CWKSP_Project::_Copy_To_File(CWKSP_Data_Item *pItem, const wxString &Direct
 	case WKSP_ITEM_Shapes    : Path.SetExt(SG_Shapes_Get_File_Extension_Default().c_str()); break;
 	case WKSP_ITEM_TIN       : Path.SetExt(SG_Shapes_Get_File_Extension_Default().c_str()); break;
 	case WKSP_ITEM_PointCloud: Path.SetExt("sg-pts-z"); break;
-	case WKSP_ITEM_Grid      : Path.SetExt(SG_Grid_Get_File_Extension_Default().c_str()); break;
+	case WKSP_ITEM_Grid      : Path.SetExt(SG_Grid_Get_File_Extension_Default  ().c_str()); break;
 	case WKSP_ITEM_Grids     : Path.SetExt("sg-gds-z"); break;
-	default:	return( false );
+	default                  : return( false );
 	}
 
 	//-----------------------------------------------------
-	wxString	Name(pItem->Get_Object()->Get_Name());
+	wxString Name(pItem->Get_Object()->Get_Name());
 
 	Name.Replace(".", "-");
 	Name.Replace(":", "-");
@@ -287,14 +271,14 @@ bool CWKSP_Project::_Copy_To_File(CWKSP_Data_Item *pItem, const wxString &Direct
 		Path.SetName(Name + wxString::Format("_%d", i));
 	}
 
-	Name	= Path.GetFullPath();
+	Name = Path.GetFullPath();
 
 	return( pItem->Get_Object()->Save(&Name) );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -304,14 +288,14 @@ bool CWKSP_Project::CopyToDB(void)
 }
 
 //---------------------------------------------------------
-bool CWKSP_Project::_Copy_To_Database(CWKSP_Data_Item *pItem, const wxString &Connection)
+bool CWKSP_Project::_Copy_To_Database(CSG_Data_Object *pObject, const wxString &Connection)
 {
 	return( true );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -437,7 +421,7 @@ bool CWKSP_Project::_Load(const wxString &File, bool bAdd, bool bUpdateMenu)
 	{
 		if( bUpdateMenu )
 		{
-			g_pData->Get_Menu_Files()->Recent_Add(SG_DATAOBJECT_TYPE_Undefined, File);
+			g_pData->Get_Menu_Files()->Recent_Add(CWKSP_Data_Menu_File::Recent_Type::Project, File);
 		}
 
 		m_File_Name	= File;
@@ -460,7 +444,7 @@ bool CWKSP_Project::_Load(const wxString &File, bool bAdd, bool bUpdateMenu)
 
 	if( bUpdateMenu )
 	{
-		g_pData->Get_Menu_Files()->Recent_Del(SG_DATAOBJECT_TYPE_Undefined, File);
+		g_pData->Get_Menu_Files()->Recent_Del(CWKSP_Data_Menu_File::Recent_Type::Project, File);
 	}
 
 	CSG_String Name(SG_File_Get_Name(&File, true));
@@ -567,7 +551,7 @@ bool CWKSP_Project::_Save(const wxString &_File, bool bSaveModified, bool bUpdat
 
 		if( bUpdateMenu )
 		{
-			g_pData->Get_Menu_Files()->Recent_Add(SG_DATAOBJECT_TYPE_Undefined, m_File_Name);
+			g_pData->Get_Menu_Files()->Recent_Add(CWKSP_Data_Menu_File::Recent_Type::Project, m_File_Name);
 		}
 
 		MSG_General_Add(_TL("Project has been saved."), true, true, SG_UI_MSG_STYLE_SUCCESS);
@@ -583,7 +567,7 @@ bool CWKSP_Project::_Save(const wxString &_File, bool bSaveModified, bool bUpdat
 
 	if( bUpdateMenu )
 	{
-		g_pData->Get_Menu_Files()->Recent_Del(SG_DATAOBJECT_TYPE_Undefined, File);
+		g_pData->Get_Menu_Files()->Recent_Del(CWKSP_Data_Menu_File::Recent_Type::Project, File);
 	}
 
 	MSG_General_Add(_TL("Could not save project."), true, true, SG_UI_MSG_STYLE_FAILURE);
@@ -595,7 +579,7 @@ bool CWKSP_Project::_Save(const wxString &_File, bool bSaveModified, bool bUpdat
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -649,7 +633,7 @@ bool CWKSP_Project::_Load_DBConnections(const CSG_MetaData &Data)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -737,10 +721,15 @@ bool CWKSP_Project::_Load_Data(const CSG_MetaData &Entry, const wxString &Projec
 	}
 
 	//-----------------------------------------------------
-	CSG_MetaData *pEntry = Entry("PARAMETERS");
+	CSG_MetaData *pEntry = Entry("PARAMETERS"); int Stretch = -1;
 
 	for(int i=0; i<pEntry->Get_Children_Count(); i++)
 	{
+		if( pEntry->Get_Child(i)->Cmp_Property("id", "STRETCH_DEFAULT") )
+		{
+			pEntry->Get_Child(i)->Get_Property("index", Stretch);
+		}
+
 		if( !pEntry->Get_Child(i)->Get_Name().CmpNoCase("DATA") && !pEntry->Get_Child(i)->Get_Content().is_Empty() && pEntry->Get_Child(i)->Get_Content().BeforeFirst(':').CmpNoCase("PGSQL") )
 		{
 			wxString File(Get_FilePath_Absolute(ProjectDir, pEntry->Get_Child(i)->Get_Content().w_str()));
@@ -749,21 +738,19 @@ bool CWKSP_Project::_Load_Data(const CSG_MetaData &Entry, const wxString &Projec
 
 		//	if( SG_Compare_SAGA_Version(Version) < 0 )
 			{
-				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_1") )
-				{
-					pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_G");
-				}
-
-				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_2") )
-				{
-					pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_B");
-				}
+				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_1") ) { pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_G"); }
+				if( pEntry->Get_Child(i)->Cmp_Property("id", "OVERLAY_2") ) { pEntry->Get_Child(i)->Set_Property("id", "OVERLAY_B"); }
 			}
 		}
 	}
 
 	pItem->Get_Parameters()->Serialize(*pEntry, false);
 
+	if( Stretch >= 0 )
+	{
+		pItem->Get_Parameters()->Set_Parameter("STRETCH_DEFAULT", Stretch);
+	}
+
 	//-----------------------------------------------------
 	if( SG_Compare_Version(Version, "7.0.0") < 0 )	// inter-version-compatibility
 	{
@@ -856,7 +843,7 @@ bool CWKSP_Project::_Save_Data(CSG_MetaData &Entry, const wxString &ProjectDir,
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -896,9 +883,9 @@ bool CWKSP_Project::_Save_Map(CSG_MetaData &Entry, const wxString &ProjectDir, C
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 //					v2.0 Compatibility					 //
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -920,7 +907,7 @@ bool CWKSP_Project::_Compatibility_Load_Data(const wxString &File)
 	CSG_String	sLine;
 	CSG_File	Stream;
 
-	if( !Stream.Open(&File, SG_FILE_R, true) )
+	if( !Stream.Open(File, SG_FILE_R, true) )
 	{
 		return( false );
 	}
@@ -1056,9 +1043,9 @@ bool CWKSP_Project::_Compatibility_Load_Map(CSG_File &Stream, const wxString &Pr
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
