@@ -61,13 +61,14 @@
 #include "res_commands.h"
 #include "res_dialogs.h"
 
+#include "wksp_layer.h"
 #include "wksp_map.h"
 
 #include "view_layout_info.h"
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -75,7 +76,7 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -96,9 +97,9 @@ const char * CVIEW_Layout_Info::Get_Item_Type_Name(int Type)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -236,7 +237,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -251,6 +252,8 @@ public:
 	{
 		m_Parameters.Add_Bool  (""           , "FRAME_SHOW"  , _TL("Frame"       ), _TL(""), true);
 		m_Parameters.Add_Int   ("FRAME_SHOW" , "FRAME_SIZE"  , _TL("Size"        ), _TL(""), 5, 2, true);
+		m_Parameters.Add_Choice("FRAME_SHOW" , "FONT_USER"   , _TL("Font"        ), _TL(""), CSG_String::Format("%s|%s", _TL("system default"), _TL("user defined")), 0);
+		m_Parameters.Add_Font  ("FONT_USER"  , "FONT"        , _TL("User Defined"), _TL(""));
 
 		m_Parameters.Add_Bool  (""           , "SCALE_FIXED" , _TL("Fixed Scale" ), _TL(""), false);
 		m_Parameters.Add_Double("SCALE_FIXED", "SCALE_NUMBER", _TL("Scale Number"), _TL(""), 10000, 0.0001, true);
@@ -259,7 +262,12 @@ public:
 	//-----------------------------------------------------
 	virtual bool		On_Parameter_Changed	(CSG_Parameters &Parameters, CSG_Parameter &Parameter)
 	{
-		Parameters.Set_Enabled("FRAME_SIZE"  , Parameters["FRAME_SHOW" ].asBool());
+		if( Parameter.Cmp_Identifier("FRAME_SHOW") )
+		{
+			Parameter.Set_Children_Enabled(Parameter.asBool());
+		}
+
+		Parameters.Set_Enabled("FONT"        , Parameters["FONT_USER"  ].asInt () == 1);
 		Parameters.Set_Enabled("SCALE_NUMBER", Parameters["SCALE_FIXED"].asBool());
 
 		return( CLayout_Item::On_Parameter_Changed(Parameters, Parameter) );
@@ -304,13 +312,24 @@ public:
 	{
 		wxRect rFrame(m_pLayout->Get_Paper2DC(m_Rect)), rMap(Get_Rect_DC()); CSG_Rect rWorld(Get_Rect_World());
 
-		m_pLayout->Get_Map()->Draw_Map(dc, rWorld, m_pLayout->Get_Paper2DC(), rMap, LAYER_DRAW_FLAG_NOEDITS);
+		m_pLayout->Get_Map()->Draw_Map(dc, rWorld, m_pLayout->Get_Paper2DC(), rMap, LAYER_DRAW_FLAG_NOEDITS, m_pLayout->Get_Map()->Get_Parameter("BG_COLOR")->asInt() ? -1 : SG_COLOR_WHITE);
 
 		if( m_Parameters["FRAME_SHOW"].asBool() )
 		{
-			dc.SetTextForeground(*wxBLACK);
+			if( m_Parameters("FONT_USER")->asInt() == 0 )
+			{
+				dc.SetTextForeground(*wxBLACK);
 
-			m_pLayout->Get_Map()->Draw_Frame(dc, rWorld, rMap, rMap.x - rFrame.x, false);
+				m_pLayout->Get_Map()->Draw_Frame(dc, rWorld, rMap, rMap.x - rFrame.x, false, true);
+			}
+			else
+			{
+				wxFont oldFont(dc.GetFont()), Font; wxColor oldColor(dc.GetTextForeground()), Color; Set_Font(m_Parameters("FONT"), Font, Color);
+
+				dc.SetFont(   Font); dc.SetTextForeground(   Color);
+				m_pLayout->Get_Map()->Draw_Frame(dc, rWorld, rMap, rMap.x - rFrame.x, false, true);
+				dc.SetFont(oldFont); dc.SetTextForeground(oldColor);
+			}
 		}
 
 		return( true );
@@ -334,7 +353,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -347,28 +366,27 @@ public:
 	CLayout_Scalebar(CVIEW_Layout_Info *pLayout)
 		: CLayout_Item(pLayout)
 	{
-		m_Parameters.Add_Choice("",
-			"UNIT"	, _TL("Unit"),
-			_TL(""),
-			CSG_String::Format("%s|%s",
-				_TL("do not show"),
-				_TL("automatically")
-			), 1
-		);
+		CSG_String Units(CSG_String::Format("%s|%s", _TL("do not show"), _TL("automatically")));
 
-		m_Parameters.Add_Choice("",
-			"STYLE"	, _TL("Style"),
-			_TL(""),
-			CSG_String::Format("%s|%s",
-				_TL("scale line"),
-				_TL("alternating scale bar")
-			), 1
-		);
+		for(int i=0; i<(int)ESG_Projection_Unit::Undefined; i++)
+		{
+			Units += "|" + CSG_Projections::Get_Unit_Name((ESG_Projection_Unit)i);
+		}
+
+		m_Parameters.Add_Choice(""         , "STYLE"    , _TL("Style"       ), _TL(""), CSG_String::Format("%s|%s", _TL("scale line"), _TL("alternating scale bar")), 1);
+		m_Parameters.Add_Choice(""         , "UNIT"     , _TL("Unit"        ), _TL(""), Units, 1);
+		m_Parameters.Add_Choice(""         , "FONT_USER", _TL("Font"        ), _TL(""), CSG_String::Format("%s|%s", _TL("system default"), _TL("user defined")), 0);
+		m_Parameters.Add_Font  ("FONT_USER", "FONT"     , _TL("User Defined"), _TL(""));
 	}
 
 	//-----------------------------------------------------
 	virtual bool		On_Parameter_Changed	(CSG_Parameters &Parameters, CSG_Parameter &Parameter)
 	{
+		if( Parameter.Cmp_Identifier("FONT_USER") )
+		{
+			Parameters.Set_Enabled("FONT"     , Parameter.asInt() == 1);
+		}
+
 		return( CLayout_Item::On_Parameter_Changed(Parameters, Parameter) );
 	}
 
@@ -392,27 +410,35 @@ public:
 		//-------------------------------------------------
 		CSG_String Unit;
 
-		if( m_Parameters("UNIT")->asInt() >= 1 )
+		if( m_Parameters("UNIT")->asInt() > 0 )
 		{
-			CSG_Projection Projection(m_pLayout->Get_Map()->Get_Projection());
+			ESG_Projection_Unit _Unit = m_Parameters("UNIT")->asInt() > 1
+				? (ESG_Projection_Unit)(m_Parameters("UNIT")->asInt() - 2) : m_pLayout->Get_Map()->Get_Projection().Get_Unit();
 
-			if( Projection.is_Okay() )
+			if( _Unit != ESG_Projection_Unit::Undefined )
 			{
-				Unit = CSG_Projections::Get_Unit_Name(Projection.Get_Unit(), true);
-
-				if( Unit.is_Empty() ) Unit = Projection.Get_Unit_Name();
-
-				if( Projection.Get_Unit() == ESG_Projection_Unit::Meter && Width > 10000. )
+				if( Width > 10000. && _Unit == ESG_Projection_Unit::Meter )
 				{
-					Unit   = CSG_Projections::Get_Unit_Name(ESG_Projection_Unit::Kilometer, true);
-
-					Width /= 1000.;
+					Width /= 1000.; _Unit = ESG_Projection_Unit::Kilometer;
 				}
+
+				Unit = CSG_Projections::Get_Unit_Name(_Unit, true);
 			}
 		}
 
 		//-------------------------------------------------
-		Draw_Scale(dc, rDC, 0., Width, SCALE_HORIZONTAL, SCALE_TICK_TOP, Style, Unit.c_str());
+		if( m_Parameters("FONT_USER")->asInt() == 0 )
+		{
+			Draw_Scale(dc, rDC, 0., Width, SCALE_HORIZONTAL, SCALE_TICK_TOP, Style, Unit.c_str());
+		}
+		else
+		{
+			wxFont oldFont(dc.GetFont()), Font; wxColor oldColor(dc.GetTextForeground()), Color; Set_Font(m_Parameters("FONT"), Font, Color);
+
+			dc.SetFont(   Font); dc.SetTextForeground(   Color);
+			Draw_Scale(dc, rDC, 0., Width, SCALE_HORIZONTAL, SCALE_TICK_TOP, Style, Unit.c_str(), true);
+			dc.SetFont(oldFont); dc.SetTextForeground(oldColor);
+		}
 
 		return( true );
 	}
@@ -420,7 +446,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -521,7 +547,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -534,12 +560,13 @@ public:
 	CLayout_Legend(CVIEW_Layout_Info *pLayout)
 		: CLayout_Item(pLayout)
 	{
-		m_Parameters.Add_Bool  (""       , "FILL"        , _TL("Fill"    ), _TL(""), false);
-		m_Parameters.Add_Color ("FILL"   , "FILL_RGB"    , _TL("Color"   ), _TL(""), SG_COLOR_WHITE);
-		m_Parameters.Add_Bool  (""       , "OUTLINE"     , _TL("Outline" ), _TL(""), false);
-		m_Parameters.Add_Color ("OUTLINE", "OUTLINE_RGB" , _TL("Color"   ), _TL(""), SG_COLOR_BLACK);
-		m_Parameters.Add_Int   ("OUTLINE", "OUTLINE_SIZE", _TL("Width"   ), _TL(""), 1, 1, true);
-		m_Parameters.Add_Int   (""       , "INFLATE"     , _TL("Distance"), _TL(""), 1, 1, true);
+		m_Parameters.Add_Bool  (""         , "FILL"        , _TL("Fill"    ), _TL(""), false);
+		m_Parameters.Add_Color ("FILL"     , "FILL_RGB"    , _TL("Color"   ), _TL(""), SG_COLOR_WHITE);
+		m_Parameters.Add_Bool  (""         , "OUTLINE"     , _TL("Outline" ), _TL(""), false);
+		m_Parameters.Add_Color ("OUTLINE"  , "OUTLINE_RGB" , _TL("Color"   ), _TL(""), SG_COLOR_BLACK);
+		m_Parameters.Add_Int   ("OUTLINE"  , "OUTLINE_SIZE", _TL("Width"   ), _TL(""), 1, 1, true);
+		m_Parameters.Add_Int   (""         , "INFLATE"     , _TL("Distance"), _TL(""), 1, 1, true);
+		m_Parameters.Add_Font  ("FONT_USER", "FONT"        , _TL("Font"    ), _TL(""));
 	}
 
 	//-----------------------------------------------------
@@ -606,7 +633,10 @@ public:
 				Scale	= rDC.GetWidth() / (double)Size.x;
 			}
 
+			wxFont oldFont(dc.GetFont()), Font; wxColor oldColor(dc.GetTextForeground()), Color; Set_Font(m_Parameters("FONT"), Font, Color);
+			dc.SetFont(   Font); dc.SetTextForeground(   Color);
 			m_pLayout->Get_Map()->Draw_Legend(dc, m_pLayout->Get_Paper2DC(), Scale, rDC.GetLeftTop());
+			dc.SetFont(oldFont); dc.SetTextForeground(oldColor);
 
 			return( true );
 		}
@@ -617,7 +647,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -747,7 +777,7 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -996,9 +1026,9 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1089,9 +1119,9 @@ protected:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1163,7 +1193,7 @@ CVIEW_Layout_Info::~CVIEW_Layout_Info(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1180,7 +1210,7 @@ int CVIEW_Layout_Info::Get_Page_Count(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1198,7 +1228,7 @@ bool CVIEW_Layout_Info::Properties(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1239,7 +1269,7 @@ wxPoint CVIEW_Layout_Info::Get_Margin_BottomRight(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1325,7 +1355,7 @@ bool CVIEW_Layout_Info::Print(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1549,7 +1579,7 @@ bool CVIEW_Layout_Info::Save(CSG_MetaData &Layout)	const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1698,7 +1728,7 @@ bool CVIEW_Layout_Info::Clipboard_Paste(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1724,7 +1754,7 @@ wxMenu * CVIEW_Layout_Info::Menu_Get_Active(void)
 	if( wxTheClipboard->IsSupported(wxDF_TEXT  )
 	||  wxTheClipboard->IsSupported(wxDF_BITMAP) )
 	{
-		CMD_Menu_Add_Item(pMenu_Add, false, ID_CMD_LAYOUT_ITEM_PASTE);
+		CMD_Menu_Add_Item(pMenu_Add, false, ID_CMD_VIEW_CLIPBOARD_PASTE);
 	}
 
 	//-----------------------------------------------------
@@ -1779,7 +1809,7 @@ wxMenu * CVIEW_Layout_Info::Menu_Get_Active(void)
 	//-----------------------------------------------------
 	else // Layout Menu...
 	{
-		CMD_Menu_Add_Item(pMenu, false, ID_CMD_LAYOUT_TO_CLIPBOARD);
+		CMD_Menu_Add_Item(pMenu, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 		pMenu->AppendSeparator();
 
@@ -1798,8 +1828,8 @@ bool CVIEW_Layout_Info::Menu_On_Command(wxCommandEvent &event)
 {
 	switch( event.GetId() )
 	{
-	case ID_CMD_LAYOUT_TO_CLIPBOARD    : Clipboard_Copy                     (); break;
-	case ID_CMD_LAYOUT_ITEM_PASTE      : Clipboard_Paste                    (); break;
+	case ID_CMD_VIEW_CLIPBOARD_COPY    : Clipboard_Copy                     (); break;
+	case ID_CMD_VIEW_CLIPBOARD_PASTE   : Clipboard_Paste                    (); break;
 
 	case ID_CMD_LAYOUT_ITEM_MAP        : Toggle_Stock_Item(Item_Type_Map     ); break;
 	case ID_CMD_LAYOUT_ITEM_LEGEND     : Toggle_Stock_Item(Item_Type_Legend  ); break;
@@ -1840,7 +1870,7 @@ bool CVIEW_Layout_Info::Menu_On_Command_UI(wxUpdateUIEvent &event)
 	case ID_CMD_LAYOUT_ITEM_MOVE_UP    : event.Enable(!m_Items.Active_is_Top   ()); break;
 	case ID_CMD_LAYOUT_ITEM_MOVE_DOWN  : event.Enable(!m_Items.Active_is_Bottom()); break;
 
-	case ID_CMD_LAYOUT_ITEM_PASTE      : event.Enable(
+	case ID_CMD_VIEW_CLIPBOARD_PASTE   : event.Enable(
 		    wxTheClipboard->IsSupported(wxDF_TEXT  )
 		||  wxTheClipboard->IsSupported(wxDF_BITMAP) );
 		break;
@@ -1851,7 +1881,7 @@ bool CVIEW_Layout_Info::Menu_On_Command_UI(wxUpdateUIEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1906,7 +1936,7 @@ bool CVIEW_Layout_Info::Draw(wxDC &dc, bool bPrintOut)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1984,9 +2014,9 @@ bool CVIEW_Layout_Info::Clipboard_Copy(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
