@@ -51,6 +51,7 @@
 #include <wx/filename.h>
 #include <wx/clipbrd.h>
 #include <wx/dataobj.h>
+#include <wx/fs_mem.h>
 
 #include <saga_gdi/sgdi_helper.h>
 
@@ -82,29 +83,29 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define LEGEND_SPACE	10
+#define LEGEND_SPACE 10
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Rect	CWKSP_Map_Extents::m_Dummy	= CSG_Rect(0, 0, 1, 1);
+CSG_Rect CWKSP_Map_Extents::m_Dummy = CSG_Rect(0, 0, 1, 1);
 
 //---------------------------------------------------------
 CWKSP_Map_Extents::CWKSP_Map_Extents(void)
 {
-	m_iExtent	= -1;
-	m_nExtents	= 0;
+	m_iExtent  = -1;
+	m_nExtents =  0;
 }
 
 //---------------------------------------------------------
@@ -173,9 +174,9 @@ bool CWKSP_Map_Extents::Add_Extent(const CSG_Rect &Extent, bool bReset)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -206,7 +207,7 @@ CWKSP_Map::~CWKSP_Map(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -215,6 +216,35 @@ wxString CWKSP_Map::Get_Name(void)
 	return( m_Name );
 }
 
+//---------------------------------------------------------
+wxString CWKSP_Map::_Set_Description_Image(void)
+{
+	wxString s;
+
+	if( Get_Manager()->Get_Parameter("PREVIEW")->asBool() )
+	{
+		int nx, ny; double x2y = Get_Extent().Get_XRange() / Get_Extent().Get_YRange();
+
+		if( x2y > 1. ) { nx = Get_Manager()->Get_Parameter("PREVIEW_SIZE")->asInt(); ny = (int)(nx / x2y); }
+		else           { ny = Get_Manager()->Get_Parameter("PREVIEW_SIZE")->asInt(); nx = (int)(ny * x2y); }
+
+		wxBitmap Thumbnail;
+
+		if( _Get_Thumbnail(Thumbnail, nx, ny) )
+		{
+			wxMemoryFSHandler::RemoveFile("preview_map.png");
+
+			wxMemoryFSHandler::AddFile("preview_map.png", Thumbnail, wxBITMAP_TYPE_PNG);
+
+			s = "<img src=\"memory:preview_map.png\" border=\"5\">";
+
+			s = "<table border=\"1\"><th><tr>" + s + "</tr></th></table><hr>";
+		}
+	}
+
+	return( s );
+}
+
 //---------------------------------------------------------
 wxString CWKSP_Map::Get_Description(void)
 {
@@ -223,6 +253,8 @@ wxString CWKSP_Map::Get_Description(void)
 	//-----------------------------------------------------
 	s += wxString::Format("<h4>%s</h4>", _TL("Map"));
 
+	s += _Set_Description_Image();
+
 	s += "<table border=\"0\">";
 
 	DESC_ADD_STR(_TL("Name"             ), m_Name.c_str());
@@ -250,9 +282,7 @@ wxString CWKSP_Map::Get_Description(void)
 //---------------------------------------------------------
 wxMenu * CWKSP_Map::Get_Menu(void)
 {
-	wxMenu *pMenu;
-
-	pMenu = new wxMenu(_TL("Map"));
+	wxMenu *pMenu = new wxMenu(_TL("Map"));
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAPS_SHOW);
@@ -261,7 +291,7 @@ wxMenu * CWKSP_Map::Get_Menu(void)
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAPS_LAYOUT_SHOW);
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAPS_SCALEBAR);
-//	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAP_NORTH_ARROW);
+	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAP_NORTH_ARROW);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAPS_GRATICULE_ADD);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAPS_BASEMAP_ADD);
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_MAPS_SYNCHRONIZE);
@@ -277,7 +307,7 @@ wxMenu * CWKSP_Map::Get_Menu(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -285,60 +315,25 @@ bool CWKSP_Map::On_Command(int Cmd_ID)
 {
 	switch( Cmd_ID )
 	{
-	default:
-		return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
+	default: return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
 
-	case ID_CMD_WKSP_ITEM_RETURN:
-		View_Show(true);
-		break;
+	case ID_CMD_WKSP_ITEM_RETURN             : View_Show     (true); break;
+	case ID_CMD_MAPS_SHOW                    : View_Toggle       (); break;
+	case ID_CMD_MAPS_3D_SHOW                 : View_3D_Toggle    (); break;
+	case ID_CMD_MAPS_LAYOUT_SHOW             : View_Layout_Toggle(); break;
 
-	case ID_CMD_MAPS_SAVE_IMAGE:
-		SaveAs_Image();
-		break;
+	case ID_CMD_MAPS_PROJECTION              : Set_Projection(); break;
+	case ID_CMD_MAPS_SCALEBAR                : Set_ScaleBar   (!is_ScaleBar   ()); break;
+	case ID_CMD_MAP_NORTH_ARROW              : Set_North_Arrow(!is_North_Arrow()); break;
+	case ID_CMD_MAPS_SYNCHRONIZE             : Set_Synchronising(!m_Parameters("SYNC_MAPS")->asBool()); break;
 
-	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:
-		SaveAs_Image_On_Change();
-		break;
+	case ID_CMD_MAPS_GRATICULE_ADD           : Add_Graticule(); break;
+	case ID_CMD_MAPS_BASEMAP_ADD             : Add_BaseMap  (); break;
 
-	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD:
-		SaveAs_Image_Clipboard(false);
-		break;
-
-	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND:
-		SaveAs_Image_Clipboard(true);
-		break;
-
-	case ID_CMD_MAPS_SCALEBAR:
-		Set_ScaleBar(!is_ScaleBar());
-		break;
-
-	case ID_CMD_MAPS_SYNCHRONIZE:
-		Set_Synchronising(!m_Parameters("SYNC_MAPS")->asBool());
-		break;
-
-	case ID_CMD_MAPS_GRATICULE_ADD:
-		Add_Graticule();
-		break;
-
-	case ID_CMD_MAPS_BASEMAP_ADD:
-		Add_BaseMap();
-		break;
-
-	case ID_CMD_MAPS_PROJECTION:
-		Set_Projection();
-		break;
-
-	case ID_CMD_MAPS_SHOW:
-		View_Toggle();
-		break;
-
-	case ID_CMD_MAPS_3D_SHOW:
-		View_3D_Toggle();
-		break;
-
-	case ID_CMD_MAPS_LAYOUT_SHOW:
-		View_Layout_Toggle();
-		break;
+	case ID_CMD_MAPS_SAVE_IMAGE              : SaveAs_Image()               ; break;
+	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE    : SaveAs_Image_On_Change()     ; break;
+	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD       : SaveAs_Image_Clipboard(false); break;
+	case ID_CMD_MAPS_SAVE_TO_CLIPBOARD_LEGEND: SaveAs_Image_Clipboard( true); break;
 	}
 
 	return( true );
@@ -349,33 +344,19 @@ bool CWKSP_Map::On_Command_UI(wxUpdateUIEvent &event)
 {
 	switch( event.GetId() )
 	{
-	default:
-		return( CWKSP_Base_Manager::On_Command_UI(event) );
+	default: return( CWKSP_Base_Manager::On_Command_UI(event) );
 
-	case ID_CMD_MAPS_SHOW:
-		event.Check(m_pView != NULL);
-		break;
+	case ID_CMD_MAPS_SHOW                    : event.Check(m_pView    != NULL); break;
+	case ID_CMD_MAPS_3D_SHOW                 : event.Check(m_pView_3D != NULL); break;
+	case ID_CMD_MAPS_LAYOUT_SHOW             : event.Check(m_pLayout  != NULL); break;
 
-	case ID_CMD_MAPS_3D_SHOW:
-		event.Check(m_pView_3D != NULL);
-		break;
+	case ID_CMD_MAPS_SCALEBAR                : event.Check(is_ScaleBar   ()); break;
+	case ID_CMD_MAP_NORTH_ARROW              : event.Check(is_North_Arrow()); break;
 
-	case ID_CMD_MAPS_LAYOUT_SHOW:
-		event.Check(m_pLayout != NULL);
-		break;
+	case ID_CMD_MAPS_GRATICULE_ADD           :
+	case ID_CMD_MAPS_BASEMAP_ADD             : event.Enable(Get_Count() > 0 && m_Projection.is_Okay()); break;
 
-	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE:
-		event.Check(is_Image_Save_Mode());
-		break;
-
-	case ID_CMD_MAPS_SCALEBAR:
-		event.Check(is_ScaleBar());
-		break;
-
-	case ID_CMD_MAPS_GRATICULE_ADD:
-	case ID_CMD_MAPS_BASEMAP_ADD:
-		event.Enable(Get_Count() > 0 && m_Projection.is_Okay());
-		break;
+	case ID_CMD_MAPS_SAVE_IMAGE_ON_CHANGE    : event.Check(is_Image_Save_Mode()); break;
 	}
 
 	return( true );
@@ -383,7 +364,7 @@ bool CWKSP_Map::On_Command_UI(wxUpdateUIEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -434,10 +415,14 @@ void CWKSP_Map::On_Create_Parameters(void)
 	);
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Bool("NODE_GENERAL",
-		"GCS_POSITION"  , _TL("Position as Geographic Coordinates"),
-		_TL("Display mouse position in status bar as geographic coordinates."),
-		false
+	m_Parameters.Add_Choice("NODE_GENERAL",
+		"GCS_POSITION"  , _TL("Display Coordinates"),
+		_TL("Show mouse position in status bar as map or geographic coordinates, the latter transformed to WGS84 and using either the sexagesimal DDD\u00B0MM'SS.S\" or the decimal degree format."),
+		CSG_String::Format("%s|%s|%s",
+			_TL("map coordinates"),
+			_TL("geographic coordinates (degree, minutes, seconds)"),
+			_TL("geographic coordinates (decimal degree)")
+		), 0
 	);
 
 	//-----------------------------------------------------
@@ -531,10 +516,23 @@ void CWKSP_Map::On_Create_Parameters(void)
 		), 1
 	);
 
-	m_Parameters.Add_Bool("SCALE_SHOW",
+	CSG_String Units(CSG_String::Format("%s|%s", _TL("do not show"), _TL("automatically")));
+
+	for(int i=0; i<(int)ESG_Projection_Unit::Undefined; i++)
+	{
+		Units += "|" + CSG_Projections::Get_Unit_Name((ESG_Projection_Unit)i);
+	}
+
+	m_Parameters.Add_Choice("SCALE_SHOW",
 		"SCALE_UNIT"    , _TL("Unit"),
 		_TL(""),
-		true
+		Units, 1
+	);
+
+	m_Parameters.Add_Double("SCALE_SHOW",
+		"SCALE_RESCALE" , _TL("Rescaling"),
+		_TL("Rescaling factor used to convert original map coordinate units, e.g. from meters to feet."),
+		1., 0., true
 	);
 
 	m_Parameters.Add_Double("SCALE_SHOW",
@@ -616,6 +614,11 @@ int CWKSP_Map::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *
 		{
 			pParameter->Set_Children_Enabled(pParameter->asBool());
 		}
+
+		if(	pParameter->Cmp_Identifier("SCALE_UNIT") )
+		{
+			pParameters->Set_Enabled("SCALE_RESCALE", pParameter->asInt() != 1); // only applies if unit is not retrieved automatically!
+		}
 	}
 
 	return( CWKSP_Base_Manager::On_Parameter_Changed(pParameters, pParameter, Flags) );
@@ -642,7 +645,7 @@ void CWKSP_Map::Parameters_Changed(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -796,11 +799,11 @@ bool CWKSP_Map::Serialize(CSG_MetaData &Root, const wxString &ProjectDir, bool b
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CWKSP_Map::Update(CWKSP_Layer *pLayer, bool bMapOnly)
+bool CWKSP_Map::Update_Map(CWKSP_Layer *pLayer, bool bMapOnly)
 {
 	if( !m_Projection.is_Okay() && pLayer->Get_Object()->Get_Projection().is_Okay() )
 	{
@@ -815,7 +818,7 @@ bool CWKSP_Map::Update(CWKSP_Layer *pLayer, bool bMapOnly)
 		{
 			CWKSP_Map_Layer *pMapLayer = (CWKSP_Map_Layer *)Get_Item(i);
 			
-			if( pMapLayer->Get_Layer()->Update(pLayer) )
+			if( pMapLayer->Get_Layer() == pLayer )
 			{
 				bRefresh = true;
 
@@ -1069,7 +1072,7 @@ CWKSP_Base_Item * CWKSP_Map::Add_Copy(CWKSP_Base_Item *pItem)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1267,7 +1270,7 @@ bool CWKSP_Map::Set_Extent(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1281,6 +1284,8 @@ void CWKSP_Map::Set_Synchronising(bool bOn)
 	if( m_Parameters("SYNC_MAPS")->asBool() != bOn )
 	{
 		m_Parameters("SYNC_MAPS")->Set_Value(bOn ? 1 : 0);
+
+		g_pActive->Update_Item(this, false);
 	}
 
 	if( bOn && m_Sync_bLock < 1 )
@@ -1317,6 +1322,8 @@ void CWKSP_Map::Set_North_Arrow(bool bOn)
 		{
 			m_pView->Do_Update();
 		}
+
+		g_pActive->Update_Item(this, false);
 	}
 }
 
@@ -1334,15 +1341,16 @@ void CWKSP_Map::Set_ScaleBar(bool bOn)
 
 		if( m_pView )
 		{
-			m_pView->Do_Update();
-			m_pView->Ruler_Refresh();
+			m_pView->Do_Update(); m_pView->Ruler_Refresh();
 		}
+
+		g_pActive->Update_Item(this, false);
 	}
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1380,7 +1388,7 @@ void CWKSP_Map::Set_CrossHair_Off(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1390,9 +1398,9 @@ void CWKSP_Map::Set_Projection(void)
 
 	if(	pTool )
 	{
-		CSG_Parameters P(*pTool->Get_Parameters());
+		CSG_Parameters P(*pTool->Get_Parameters()); P.Set_Name(_TL("Map CRS"));
 
-		P.Set_Parameter("CRS_STRING"   , m_Projection.Get_WKT());
+		P.Set_Parameter("CRS_STRING", m_Projection.Get_WKT());
 
 		P.Add_Bool("", "ONTHEFLY", _TL("On-The-Fly Projection"),
 			_TL("Turn on the on-the-fly projection for all layers in the map."),
@@ -1404,7 +1412,7 @@ void CWKSP_Map::Set_Projection(void)
 		{
 			CSG_Projection Projection; CSG_Parameters &_P = *pTool->Get_Parameters();
 
-			Projection.Create(_P["CRS_WKT"].asString());
+			Projection.Create(_P["CRS_WKT"].asString(), _P["CRS_PROJ"].asString());
 
 			if( P("ONTHEFLY")->asBool() )
 			{
@@ -1434,7 +1442,7 @@ void CWKSP_Map::Set_Projection(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1472,7 +1480,7 @@ void CWKSP_Map::View_Refresh(bool bMapOnly)
 		}
 	}
 
-	_Set_Thumbnail();
+	_Set_Thumbnail(m_Thumbnail);
 
 	if( g_pMap_Buttons ) { g_pMap_Buttons->Refresh(); }
 }
@@ -1545,7 +1553,7 @@ void CWKSP_Map::View_Layout_Toggle(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1585,7 +1593,7 @@ CSG_Point CWKSP_Map::Get_World(wxRect rClient, wxPoint Point)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2111,35 +2119,43 @@ void CWKSP_Map::_Img_Save(wxString file, int type)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-const wxBitmap & CWKSP_Map::Get_Thumbnail(int dx, int dy)
+const wxBitmap & CWKSP_Map::Get_Thumbnail(int nx, int ny)
 {
-	if( dx > 0 && dy > 0 && (!m_Thumbnail.IsOk() || m_Thumbnail.GetWidth() != dx || m_Thumbnail.GetHeight() != dy) )
-	{
-		m_Thumbnail.Create(dx, dy);
-
-		_Set_Thumbnail();
-	}
+	_Get_Thumbnail(m_Thumbnail, nx, ny);
 
 	return( m_Thumbnail );
 }
 
 //---------------------------------------------------------
-bool CWKSP_Map::_Set_Thumbnail(void)
+bool CWKSP_Map::_Get_Thumbnail(wxBitmap &Thumbnail, int nx, int ny)
 {
-	if( m_Thumbnail.IsOk() && m_Thumbnail.GetWidth() > 0 && m_Thumbnail.GetHeight() > 0 )
+	if( nx > 0 && ny > 0 )
 	{
-		wxMemoryDC		dc;
-		wxRect			r(0, 0, m_Thumbnail.GetWidth(), m_Thumbnail.GetHeight());
+		if( !Thumbnail.IsOk() || Thumbnail.GetWidth() != nx || Thumbnail.GetHeight() != ny )
+		{
+			Thumbnail.Create(nx, ny);
+		}
 
-		dc.SelectObject(m_Thumbnail);
-		dc.SetBackground(*wxWHITE_BRUSH);
-		dc.Clear();
+		return( _Set_Thumbnail(Thumbnail) );
+	}
 
-		Draw_Map(dc, Get_Extent(), 1., r, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS);
+	return( false );
+}
+
+//---------------------------------------------------------
+bool CWKSP_Map::_Set_Thumbnail(wxBitmap &Thumbnail)
+{
+	if( Thumbnail.IsOk() && Thumbnail.GetWidth() > 0 && Thumbnail.GetHeight() > 0 )
+	{
+		wxMemoryDC dc; wxRect r(0, 0, Thumbnail.GetWidth(), Thumbnail.GetHeight());
+
+		dc.SelectObject(Thumbnail); dc.SetBackground(Get_Background()); dc.Clear();
+
+		Draw_Map(dc, Get_Extent(), 1., r, LAYER_DRAW_FLAG_NOEDITS|LAYER_DRAW_FLAG_NOLABELS|LAYER_DRAW_FLAG_THUMBNAIL);
 
 		dc.SelectObject(wxNullBitmap);
 
@@ -2151,7 +2167,7 @@ bool CWKSP_Map::_Set_Thumbnail(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2162,7 +2178,7 @@ wxColour CWKSP_Map::Get_Background(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2190,7 +2206,7 @@ void CWKSP_Map::Draw_Map(CSG_Map_DC &dc, int Flags)
 	{
 		switch( Get_Item(i)->Get_Type() )
 		{
-		case WKSP_ITEM_Map_Layer:
+		case WKSP_ITEM_Map_Layer    :
 		{
 			CWKSP_Map_Layer     *pLayer	= (CWKSP_Map_Layer     *)Get_Item(i);
 
@@ -2201,7 +2217,7 @@ void CWKSP_Map::Draw_Map(CSG_Map_DC &dc, int Flags)
 		}
 		break;
 
-		case WKSP_ITEM_Map_Graticule:
+		case WKSP_ITEM_Map_Graticule: if( (Flags & LAYER_DRAW_FLAG_THUMBNAIL) == 0 )
 		{
 			CWKSP_Map_Graticule *pLayer	= (CWKSP_Map_Graticule *)Get_Item(i);
 
@@ -2212,7 +2228,7 @@ void CWKSP_Map::Draw_Map(CSG_Map_DC &dc, int Flags)
 		}
 		break;
 
-		case WKSP_ITEM_Map_BaseMap:
+		case WKSP_ITEM_Map_BaseMap  : if( (Flags & LAYER_DRAW_FLAG_THUMBNAIL) == 0 )
 		{
 			CWKSP_Map_BaseMap   *pLayer	= (CWKSP_Map_BaseMap   *)Get_Item(i);
 
@@ -2229,9 +2245,12 @@ void CWKSP_Map::Draw_Map(CSG_Map_DC &dc, int Flags)
 	}
 
 	//-----------------------------------------------------
-	Draw_Extent     (dc);
-	Draw_ScaleBar   (dc);
-	Draw_North_Arrow(dc);
+	if( (Flags & LAYER_DRAW_FLAG_THUMBNAIL) == 0 )
+	{
+		Draw_Extent     (dc);
+		Draw_ScaleBar   (dc);
+		Draw_North_Arrow(dc);
+	}
 }
 
 //---------------------------------------------------------
@@ -2240,37 +2259,39 @@ void CWKSP_Map::Draw_Frame(wxDC &dc, wxRect rMap, int Width)
 	Draw_Frame(dc, Get_World(rMap), rMap, Width, is_ScaleBar(true));
 }
 
-void CWKSP_Map::Draw_Frame(wxDC &dc, const CSG_Rect &rWorld, wxRect rMap, int Width, bool bScaleBar)
+void CWKSP_Map::Draw_Frame(wxDC &dc, const CSG_Rect &rWorld, wxRect rMap, int Width, bool bScaleBar, bool bUseDCFont)
 {
 	Draw_Edge(dc, EDGE_STYLE_SIMPLE, rMap.GetLeft(), rMap.GetTop(), rMap.GetRight(), rMap.GetBottom());
 
-	wxRect	rFrame(rMap);	rFrame.Inflate(Width);
+	wxRect rFrame(rMap); rFrame.Inflate(Width);
 
 	Draw_Scale(dc, wxRect(rMap  .GetLeft(), rFrame.GetTop(), rMap.GetWidth (), Width),
-		rWorld.Get_XMin(), rWorld.Get_XMax(),  true,  true, false
+		rWorld.Get_XMin(), rWorld.Get_XMax(),  true,  true, false, "", bUseDCFont
 	);
 
 	Draw_Scale(dc, wxRect(rFrame.GetLeft(), rMap  .GetTop(), Width, rMap.GetHeight()),
-		rWorld.Get_YMin(), rWorld.Get_YMax(), false, false, false
+		rWorld.Get_YMin(), rWorld.Get_YMax(), false, false, false, "", bUseDCFont
 	);
 
 	if( bScaleBar == false )
 	{
 		Draw_Scale(dc, wxRect(rMap.GetLeft (), rMap.GetBottom(), rMap.GetWidth (), Width),
-			rWorld.Get_XMin(), rWorld.Get_XMax(),  true,  true, true
+			rWorld.Get_XMin(), rWorld.Get_XMax(),  true,  true, true, "", bUseDCFont
 		);
 
 		Draw_Scale(dc, wxRect(rMap.GetRight(), rMap.GetTop   (), Width, rMap.GetHeight()),
-			rWorld.Get_YMin(), rWorld.Get_YMax(), false, false, true
+			rWorld.Get_YMin(), rWorld.Get_YMax(), false, false, true, "", bUseDCFont
 		);
 	}
 	else
 	{
 		Draw_Scale(dc, wxRect(rMap.GetLeft (), rMap.GetBottom(), rMap.GetWidth (), Width),
-			0., rWorld.Get_XRange(),  true,  true, true);
+			0., rWorld.Get_XRange(),  true,  true, true, "", bUseDCFont
+		);
 
 		Draw_Scale(dc, wxRect(rMap.GetRight(), rMap.GetTop   (), Width, rMap.GetHeight()),
-			0., rWorld.Get_YRange(), false, false, true);
+			0., rWorld.Get_YRange(), false, false, true, "", bUseDCFont
+		);
 	}
 
 	Draw_Edge(dc, EDGE_STYLE_SIMPLE, rFrame.GetLeft(), rFrame.GetTop(), rFrame.GetRight(), rFrame.GetBottom());
@@ -2396,8 +2417,6 @@ bool CWKSP_Map::Draw_ScaleBar(CSG_Map_DC &dc)
 		return( true );
 	}
 
-	double dWidth = 0.01 * m_Parameters("SCALE_WIDTH")->asDouble();
-
 	wxRect r = !m_Parameters("SCALE_EXTENT")->asBool() ? wxRect(dc.rDC()) : wxRect(
 		(int) dc.xWorld2DC    (Get_Extent().Get_XMin  ()),
 		(int) dc.yWorld2DC    (Get_Extent().Get_YMax  ()),
@@ -2408,23 +2427,32 @@ bool CWKSP_Map::Draw_ScaleBar(CSG_Map_DC &dc)
 	r = wxRect(
 		(int)(0.5 + r.GetWidth () * 0.01 * m_Parameters("SCALE_OFFSET_X")->asDouble()) + r.GetX(), r.GetY() + r.GetHeight() -
 		(int)(0.5 + r.GetHeight() * 0.01 * m_Parameters("SCALE_OFFSET_Y")->asDouble()),
-		(int)(0.5 + r.GetWidth () * dWidth),
+		(int)(0.5 + r.GetWidth () * 0.01 * m_Parameters("SCALE_WIDTH"   )->asDouble()),
 		(int)(0.5 + r.GetHeight() * 0.01 * m_Parameters("SCALE_HEIGHT"  )->asDouble())
 	);
 
-	dWidth = dc.DC2World() * r.GetWidth();
+	double Length = dc.DC2World() * r.GetWidth();
+
+	if( m_Parameters("SCALE_UNIT")->asInt() != 1 && m_Parameters("SCALE_RESCALE")->asDouble() > 0. ) // don't rescale if unit is retrieved from coordinate system (automatically)!
+	{
+		Length *= m_Parameters("SCALE_RESCALE")->asDouble();
+	}
 
 	CSG_String Unit;
 
-	if( m_Projection.is_Okay() && m_Parameters("SCALE_UNIT")->asBool() )
+	if( m_Parameters("SCALE_UNIT")->asInt() > 0 )
 	{
-		Unit = CSG_Projections::Get_Unit_Name(m_Projection.Get_Unit(), true);
+		ESG_Projection_Unit _Unit = m_Parameters("SCALE_UNIT")->asInt() >= 2
+			? (ESG_Projection_Unit)(m_Parameters("SCALE_UNIT")->asInt()  - 2) : m_Projection.Get_Unit();
 
-		if( Unit.is_Empty() ) Unit = m_Projection.Get_Unit_Name();
-
-		if( m_Projection.Get_Unit() == ESG_Projection_Unit::Meter && dWidth > 10000. )
+		if( _Unit != ESG_Projection_Unit::Undefined )
 		{
-			dWidth /= 1000.; Unit = CSG_Projections::Get_Unit_Name(ESG_Projection_Unit::Kilometer, true);
+			if( Length > 10000. && _Unit == ESG_Projection_Unit::Meter )
+			{
+				Length /= 1000.; _Unit = ESG_Projection_Unit::Kilometer;
+			}
+
+			Unit = CSG_Projections::Get_Unit_Name(_Unit, true);
 		}
 	}
 
@@ -2435,7 +2463,7 @@ bool CWKSP_Map::Draw_ScaleBar(CSG_Map_DC &dc)
 		Style |= SCALE_STYLE_BLACKWHITE;
 	}
 
-	Draw_Scale(dc.Get_DC(), r, 0., dWidth, SCALE_HORIZONTAL, SCALE_TICK_TOP, Style, Unit.c_str());
+	Draw_Scale(dc.Get_DC(), r, 0., Length, SCALE_HORIZONTAL, SCALE_TICK_TOP, Style, Unit.c_str());
 
 	return( true );
 }
@@ -2450,23 +2478,30 @@ bool CWKSP_Map::Draw_Extent(CSG_Map_DC &dc)
 
 	if( dc.Draw_Image_Begin(m_Parameters("SEL_TRANSP")->asDouble() / 100.) )
 	{
-		int Color = m_Parameters("SEL_COLOUR")->asColor();
+		int Color = m_Parameters("SEL_COLOUR")->asColor(); wxRect r(dc.rDC()); r.Inflate(1);
 
-		wxRect r(dc.rDC()); r.Inflate(1);
+		double dx = dc.rWorld().Get_XRange() - Get_Extent().Get_XRange();
+		double dy = dc.rWorld().Get_YRange() - Get_Extent().Get_YRange();
 
-		if( dc.rWorld().Get_XRange() > Get_Extent().Get_XRange() )
+		if( dx > dy )
 		{
-			int d = (int)(0.5 + dc.World2DC() * (dc.rWorld().Get_XRange() - Get_Extent().Get_XRange()) / 2.);
+			int d = (int)(dx * dc.World2DC() / 2.);
 
-			dc.Draw_Image_Pixels(r.GetLeft (), r.GetTop(), r.GetLeft () + d, r.GetBottom(), Color);
-			dc.Draw_Image_Pixels(r.GetRight(), r.GetTop(), r.GetRight() - d, r.GetBottom(), Color);
+			if( d > 0 )
+			{
+				dc.Draw_Image_Pixels(r.GetLeft (), r.GetTop(), r.GetLeft () + d, r.GetBottom(), Color);
+				dc.Draw_Image_Pixels(r.GetRight(), r.GetTop(), r.GetRight() - d, r.GetBottom(), Color);
+			}
 		}
-		else
+		else // if( dx < dy )
 		{
-			int d = (int)(0.5 + dc.World2DC() * (dc.rWorld().Get_YRange() - Get_Extent().Get_YRange()) / 2.);
+			int d = (int)(dy * dc.World2DC() / 2.);
 
-			dc.Draw_Image_Pixels(r.GetLeft(), r.GetTop   (), r.GetRight(), r.GetTop   () + d, Color);
-			dc.Draw_Image_Pixels(r.GetLeft(), r.GetBottom(), r.GetRight(), r.GetBottom() - d, Color);
+			if( d > 0 )
+			{
+				dc.Draw_Image_Pixels(r.GetLeft(), r.GetTop   (), r.GetRight(), r.GetTop   () + d, Color);
+				dc.Draw_Image_Pixels(r.GetLeft(), r.GetBottom(), r.GetRight(), r.GetBottom() - d, Color);
+			}
 		}
 
 		dc.Draw_Image_End();
@@ -2477,7 +2512,7 @@ bool CWKSP_Map::Draw_Extent(CSG_Map_DC &dc)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -2523,9 +2558,9 @@ void CWKSP_Map::Show_Coordinate(const CSG_Point &Coordinate) const
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
