@@ -107,15 +107,9 @@ CGeoref_Grid::CGeoref_Grid(bool bList)
 	);
 
 	//-----------------------------------------------------
-	Parameters.Add_Choice("",
+	Parameters.Add_Grid_Resampling("",
 		"RESAMPLING", _TL("Resampling"),
-		_TL(""),
-		CSG_String::Format("%s|%s|%s|%s",
-			_TL("Nearest Neighbour"),
-			_TL("Bilinear Interpolation"),
-			_TL("Bicubic Spline Interpolation"),
-			_TL("B-Spline Interpolation")
-		), 3
+		_TL("")
 	);
 
 	Parameters.Add_Bool("",
@@ -475,19 +469,11 @@ bool CGeoref_Grid::Rectify(void)
 	}
 
 	//-----------------------------------------------------
-	TSG_Grid_Resampling Resampling;
-
-	switch( Parameters("RESAMPLING")->asInt() )
-	{
-	default: Resampling = GRID_RESAMPLING_NearestNeighbour; break;
-	case  1: Resampling = GRID_RESAMPLING_Bilinear        ; break;
-	case  2: Resampling = GRID_RESAMPLING_BicubicSpline   ; break;
-	case  3: Resampling = GRID_RESAMPLING_BSpline         ; break;
-	}
+	CSG_Grid_Resampling Resampling = Parameters("RESAMPLING")->asGrid_Resampling();
 
 	bool bBytewise = Parameters("BYTEWISE")->asBool();
 
-	TSG_Data_Type Type = (Resampling == GRID_RESAMPLING_NearestNeighbour || Parameters("BYTEWISE")->asBool())
+	TSG_Data_Type Type = (Resampling == CSG_Grid_Resampling::NearestNeighbour || Parameters("BYTEWISE")->asBool())
 		? SG_DATATYPE_Undefined : Parameters("DATA_TYPE")->asDataType()->Get_Data_Type();
 
 	//-----------------------------------------------------
@@ -547,7 +533,7 @@ bool CGeoref_Grid::Rectify(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-bool CGeoref_Grid::Rectify(CSG_Array_Pointer &Sources, CSG_Array_Pointer &Targets, TSG_Grid_Resampling Resampling, bool bBytewise)
+bool CGeoref_Grid::Rectify(CSG_Array_Pointer &Sources, CSG_Array_Pointer &Targets, CSG_Grid_Resampling Resampling, bool bBytewise)
 {
 	if( !Sources.Get_Size() || Sources.Get_Size() < Targets.Get_Size() )
 	{
